@@ -44,6 +44,7 @@
 
 /*!
     \class ExtensionSystem::PluginView
+    \inmodule QtCreator
     \brief The PluginView class implements a widget that shows a list of all
     plugins and their state.
 
@@ -56,17 +57,22 @@
 */
 
 /*!
-    \fn void PluginView::currentPluginChanged(ExtensionSystem::PluginSpec *spec)
+    \fn void ExtensionSystem::PluginView::currentPluginChanged(ExtensionSystem::PluginSpec *spec)
     The current selection in the plugin list has changed to the
     plugin corresponding to \a spec.
 */
 
 /*!
-    \fn void PluginView::pluginActivated(ExtensionSystem::PluginSpec *spec)
+    \fn void ExtensionSystem::PluginView::pluginActivated(ExtensionSystem::PluginSpec *spec)
     The plugin list entry corresponding to \a spec has been activated,
     for example by a double-click.
 */
 
+/*!
+    \fn void ExtensionSystem::PluginView::pluginSettingsChanged(ExtensionSystem::PluginSpec *spec)
+    The settings for the plugin list entry corresponding to \a spec changed.
+*/
+
 Q_DECLARE_METATYPE(ExtensionSystem::PluginSpec*)
 
 using namespace Utils;
@@ -212,10 +218,12 @@ public:
 class CollectionItem : public TreeItem
 {
 public:
-    CollectionItem(const QString &name, QList<PluginSpec *> plugins, PluginView *view)
-        : m_name(name), m_plugins(plugins), m_view(view)
+    CollectionItem(const QString &name, const QVector<PluginSpec *> &plugins, PluginView *view)
+        : m_name(name)
+        , m_plugins(plugins)
+        , m_view(view)
     {
-        foreach (PluginSpec *spec, plugins)
+        for (PluginSpec *spec : plugins)
             appendChild(new PluginItem(spec, view));
     }
 
@@ -235,7 +243,7 @@ public:
                 return PluginView::tr("Load on Startup");
             if (role == Qt::CheckStateRole || role == SortRole) {
                 int checkedCount = 0;
-                foreach (PluginSpec *spec, m_plugins) {
+                for (PluginSpec *spec : m_plugins) {
                     if (spec->isEnabledBySettings())
                         ++checkedCount;
                 }
@@ -254,9 +262,11 @@ public:
     bool setData(int column, const QVariant &data, int role) override
     {
         if (column == LoadedColumn && role == Qt::CheckStateRole) {
-            const QList<PluginSpec *> affectedPlugins =
-                    Utils::filtered(m_plugins, [](PluginSpec *spec) { return !spec->isRequired(); });
-            if (m_view->setPluginsEnabled(affectedPlugins.toSet(), data.toBool())) {
+            const QVector<PluginSpec *> affectedPlugins
+                = Utils::filtered(m_plugins, [](PluginSpec *spec) { return !spec->isRequired(); });
+            if (m_view->setPluginsEnabled(Utils::transform<QSet>(affectedPlugins,
+                                                                 [](PluginSpec *s) { return s; }),
+                                          data.toBool())) {
                 update();
                 return true;
             }
@@ -274,7 +284,7 @@ public:
 
 public:
     QString m_name;
-    QList<PluginSpec *> m_plugins;
+    const QVector<PluginSpec *> m_plugins;
     PluginView *m_view; // Not owned.
 };
 
@@ -317,8 +327,8 @@ private:
 using namespace ExtensionSystem::Internal;
 
 /*!
-    Constructs a PluginView that gets the list of plugins from the
-    given plugin \a manager with a given \a parent widget.
+    Constructs a plugin view with \a parent that displays a list of plugins
+    from a plugin manager.
 */
 PluginView::PluginView(QWidget *parent)
     : QWidget(parent)
@@ -379,18 +389,27 @@ PluginSpec *PluginView::currentPlugin() const
     return pluginForIndex(m_categoryView->currentIndex());
 }
 
+/*!
+    Sets the \a filter for listing plugins.
+*/
 void PluginView::setFilter(const QString &filter)
 {
     m_sortModel->setFilterFixedString(filter);
     m_categoryView->expandAll();
 }
 
+/*!
+    Sets the list filtering to \a showHidden.
+*/
 void PluginView::setShowHidden(bool showHidden)
 {
     m_sortModel->setShowHidden(showHidden);
     m_categoryView->expandAll();
 }
 
+/*!
+    Returns whether hidden plugins are listed.
+*/
 bool PluginView::isShowingHidden() const
 {
     return m_sortModel->isShowingHidden();
@@ -408,17 +427,17 @@ void PluginView::updatePlugins()
     // Model.
     m_model->clear();
 
-
-    QList<CollectionItem *> collections;
-    const QHash<QString, QList<PluginSpec *>> pluginCollections = PluginManager::pluginCollections();
+    const QHash<QString, QVector<PluginSpec *>> pluginCollections
+        = PluginManager::pluginCollections();
+    std::vector<CollectionItem *> collections;
     const auto end = pluginCollections.cend();
     for (auto it = pluginCollections.cbegin(); it != end; ++it) {
         const QString name = it.key().isEmpty() ? tr("Utilities") : it.key();
-        collections.append(new CollectionItem(name, it.value(), this));
+        collections.push_back(new CollectionItem(name, it.value(), this));
     }
     Utils::sort(collections, &CollectionItem::m_name);
 
-    foreach (CollectionItem *collection, collections)
+    for (CollectionItem *collection : qAsConst(collections))
         m_model->rootItem()->appendChild(collection);
 
     emit m_model->layoutChanged();
@@ -436,8 +455,8 @@ bool PluginView::setPluginsEnabled(const QSet<PluginSpec *> &plugins, bool enabl
 {
     QSet<PluginSpec *> additionalPlugins;
     if (enable) {
-        foreach (PluginSpec *spec, plugins) {
-            foreach (PluginSpec *other, PluginManager::pluginsRequiredByPlugin(spec)) {
+        for (PluginSpec *spec : plugins) {
+            for (PluginSpec *other : PluginManager::pluginsRequiredByPlugin(spec)) {
                 if (!other->isEnabledBySettings())
                     additionalPlugins.insert(other);
             }
@@ -453,8 +472,8 @@ bool PluginView::setPluginsEnabled(const QSet<PluginSpec *> &plugins, bool enabl
                 return false;
         }
     } else {
-        foreach (PluginSpec *spec, plugins) {
-            foreach (PluginSpec *other, PluginManager::pluginsRequiringPlugin(spec)) {
+        for (PluginSpec *spec : plugins) {
+            for (PluginSpec *other : PluginManager::pluginsRequiringPlugin(spec)) {
                 if (other->isEnabledBySettings())
                     additionalPlugins.insert(other);
             }
@@ -471,8 +490,8 @@ bool PluginView::setPluginsEnabled(const QSet<PluginSpec *> &plugins, bool enabl
         }
     }
 
-    QSet<PluginSpec *> affectedPlugins = plugins + additionalPlugins;
-    foreach (PluginSpec *spec, affectedPlugins) {
+    const QSet<PluginSpec *> affectedPlugins = plugins + additionalPlugins;
+    for (PluginSpec *spec : affectedPlugins) {
         PluginItem *item = m_model->findItemAtLevel<2>([spec](PluginItem *item) {
                 return item->m_spec == spec;
         });
