@@ -27,6 +27,7 @@
 
 #include "environment.h"
 #include "qtcassert.h"
+#include "macroexpander.h"
 
 #include "synchronousprocess.h"
 #include "hostosinfo.h"
@@ -49,13 +50,15 @@
     This class has some validation logic for embedding into QWizardPage.
 */
 
+namespace Utils {
+
 static QString appBundleExpandedPath(const QString &path)
 {
-    if (Utils::HostOsInfo::hostOs() == Utils::OsTypeMac && path.endsWith(QLatin1String(".app"))) {
+    if (HostOsInfo::hostOs() == OsTypeMac && path.endsWith(".app")) {
         // possibly expand to Foo.app/Contents/MacOS/Foo
         QFileInfo info(path);
         if (info.isDir()) {
-            QString exePath = path + QLatin1String("/Contents/MacOS/") + info.completeBaseName();
+            QString exePath = path + "/Contents/MacOS/" + info.completeBaseName();
             if (QFileInfo::exists(exePath))
                 return exePath;
         }
@@ -63,9 +66,7 @@ static QString appBundleExpandedPath(const QString &path)
     return path;
 }
 
-Utils::PathChooser::AboutToShowContextMenuHandler Utils::PathChooser::s_aboutToShowContextMenuHandler;
-
-namespace Utils {
+PathChooser::AboutToShowContextMenuHandler PathChooser::s_aboutToShowContextMenuHandler;
 
 // ------------------ BinaryVersionToolTipEventFilter
 // Event filter to be installed on a lineedit used for entering
@@ -78,12 +79,12 @@ class BinaryVersionToolTipEventFilter : public QObject
 public:
     explicit BinaryVersionToolTipEventFilter(QLineEdit *le);
 
-    virtual bool eventFilter(QObject *, QEvent *);
+    bool eventFilter(QObject *, QEvent *) override;
 
     QStringList arguments() const { return m_arguments; }
     void setArguments(const QStringList &arguments) { m_arguments = arguments; }
 
-    static QString toolVersion(const QString &binary, const QStringList &arguments);
+    static QString toolVersion(const CommandLine &cmd);
 
 private:
     // Extension point for concatenating existing tooltips.
@@ -102,37 +103,38 @@ bool BinaryVersionToolTipEventFilter::eventFilter(QObject *o, QEvent *e)
 {
     if (e->type() != QEvent::ToolTip)
         return false;
-    QLineEdit *le = qobject_cast<QLineEdit *>(o);
+    auto le = qobject_cast<QLineEdit *>(o);
     QTC_ASSERT(le, return false);
 
     const QString binary = le->text();
     if (!binary.isEmpty()) {
-        const QString version = BinaryVersionToolTipEventFilter::toolVersion(QDir::cleanPath(binary), m_arguments);
+        const QString version = BinaryVersionToolTipEventFilter::toolVersion(
+                    CommandLine(QDir::cleanPath(binary), m_arguments));
         if (!version.isEmpty()) {
             // Concatenate tooltips.
-            QString tooltip = QLatin1String("<html><head/><body>");
+            QString tooltip = "<html><head/><body>";
             const QString defaultValue = defaultToolTip();
             if (!defaultValue.isEmpty()) {
-                tooltip += QLatin1String("<p>");
+                tooltip += "<p>";
                 tooltip += defaultValue;
-                tooltip += QLatin1String("</p>");
+                tooltip += "</p>";
             }
-            tooltip += QLatin1String("<pre>");
+            tooltip += "<pre>";
             tooltip += version;
-            tooltip += QLatin1String("</pre><body></html>");
+            tooltip += "</pre><body></html>";
             le->setToolTip(tooltip);
         }
     }
     return false;
 }
 
-QString BinaryVersionToolTipEventFilter::toolVersion(const QString &binary, const QStringList &arguments)
+QString BinaryVersionToolTipEventFilter::toolVersion(const CommandLine &cmd)
 {
-    if (binary.isEmpty())
+    if (cmd.executable().isEmpty())
         return QString();
     SynchronousProcess proc;
     proc.setTimeoutS(1);
-    SynchronousProcessResponse response = proc.runBlocking(binary, arguments);
+    SynchronousProcessResponse response = proc.runBlocking(cmd);
     if (response.result != SynchronousProcessResponse::Finished)
         return QString();
     return response.allOutput();
@@ -147,10 +149,10 @@ public:
         BinaryVersionToolTipEventFilter(pe->lineEdit()), m_pathChooser(pe) {}
 
 private:
-    virtual QString defaultToolTip() const
+    QString defaultToolTip() const override
         { return m_pathChooser->errorMessage(); }
 
-    const PathChooser *m_pathChooser;
+    const PathChooser *m_pathChooser = nullptr;
 };
 
 // ------------------ PathChooserPrivate
@@ -162,24 +164,24 @@ public:
 
     QString expandedPath(const QString &path) const;
 
-    QHBoxLayout *m_hLayout;
-    FancyLineEdit *m_lineEdit;
+    QHBoxLayout *m_hLayout = nullptr;
+    FancyLineEdit *m_lineEdit = nullptr;
 
-    PathChooser::Kind m_acceptingKind;
+    PathChooser::Kind m_acceptingKind = PathChooser::ExistingDirectory;
     QString m_dialogTitleOverride;
     QString m_dialogFilter;
     QString m_initialBrowsePathOverride;
-    QString m_baseDirectory;
+    FilePath m_baseDirectory;
     Environment m_environment;
-    BinaryVersionToolTipEventFilter *m_binaryVersionToolTipEventFilter;
+    BinaryVersionToolTipEventFilter *m_binaryVersionToolTipEventFilter = nullptr;
     QList<QAbstractButton *> m_buttons;
+    MacroExpander *m_macroExpander;
 };
 
 PathChooserPrivate::PathChooserPrivate() :
     m_hLayout(new QHBoxLayout),
     m_lineEdit(new FancyLineEdit),
-    m_acceptingKind(PathChooser::ExistingDirectory),
-    m_binaryVersionToolTipEventFilter(0)
+    m_macroExpander(globalMacroExpander())
 {
 }
 
@@ -187,14 +189,19 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
 {
     if (input.isEmpty())
         return input;
-    const QString path = QDir::cleanPath(m_environment.expandVariables(input));
+
+    QString expandedInput = m_environment.expandVariables(input);
+    if (m_macroExpander)
+        expandedInput = m_macroExpander->expand(expandedInput);
+
+    const QString path = FilePath::fromUserInput(expandedInput).toString();
     if (path.isEmpty())
         return path;
 
     switch (m_acceptingKind) {
     case PathChooser::Command:
     case PathChooser::ExistingCommand: {
-        const FileName expanded = m_environment.searchInPath(path, QStringList(m_baseDirectory));
+        const FilePath expanded = m_environment.searchInPath(path, {m_baseDirectory});
         return expanded.isEmpty() ? path : expanded.toString();
     }
     case PathChooser::Any:
@@ -204,7 +211,7 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
     case PathChooser::File:
     case PathChooser::SaveFile:
         if (!m_baseDirectory.isEmpty() && QFileInfo(path).isRelative())
-            return QFileInfo(m_baseDirectory + QLatin1Char('/') + path).absoluteFilePath();
+            return m_baseDirectory.pathAppended(path).toFileInfo().absoluteFilePath();
         break;
     }
     return path;
@@ -224,10 +231,9 @@ PathChooser::PathChooser(QWidget *parent) :
             [this] { emit rawPathChanged(rawPath()); });
     connect(d->m_lineEdit, &FancyLineEdit::validChanged, this, &PathChooser::validChanged);
     connect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
-    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] { emit pathChanged(path()); });
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] { emit pathChanged(d->m_lineEdit->text()); });
 
     d->m_lineEdit->setMinimumWidth(120);
-    d->m_lineEdit->setErrorColor(creatorTheme()->color(Theme::TextColorError));
     d->m_hLayout->addWidget(d->m_lineEdit);
     d->m_hLayout->setSizeConstraint(QLayout::SetMinimumSize);
 
@@ -274,30 +280,19 @@ QAbstractButton *PathChooser::buttonAtIndex(int index) const
     return d->m_buttons.at(index);
 }
 
-QString PathChooser::baseDirectory() const
+void PathChooser::setBaseDirectory(const FilePath &base)
+{
+    if (d->m_baseDirectory == base)
+        return;
+    d->m_baseDirectory = base;
+    triggerChanged();
+}
+
+FilePath PathChooser::baseDirectory() const
 {
     return d->m_baseDirectory;
 }
 
-void PathChooser::setBaseDirectory(const QString &directory)
-{
-    if (d->m_baseDirectory == directory)
-        return;
-    d->m_baseDirectory = directory;
-    triggerChanged();
-}
-
-FileName PathChooser::baseFileName() const
-{
-    return FileName::fromString(d->m_baseDirectory);
-}
-
-void PathChooser::setBaseFileName(const FileName &base)
-{
-    d->m_baseDirectory = base.toString();
-    triggerChanged();
-}
-
 void PathChooser::setEnvironment(const Environment &env)
 {
     QString oldExpand = path();
@@ -318,14 +313,14 @@ QString PathChooser::path() const
     return fileName().toString();
 }
 
-FileName PathChooser::rawFileName() const
+FilePath PathChooser::rawFileName() const
 {
-    return FileName::fromString(QDir::fromNativeSeparators(d->m_lineEdit->text()));
+    return FilePath::fromString(QDir::fromNativeSeparators(d->m_lineEdit->text()));
 }
 
-FileName PathChooser::fileName() const
+FilePath PathChooser::fileName() const
 {
-    return FileName::fromUserInput(d->expandedPath(rawFileName().toString()));
+    return FilePath::fromUserInput(d->expandedPath(rawFileName().toString()));
 }
 
 // FIXME: try to remove again
@@ -338,28 +333,18 @@ QString PathChooser::expandedDirectory(const QString &input, const Environment &
     if (path.isEmpty())
         return path;
     if (!baseDir.isEmpty() && QFileInfo(path).isRelative())
-        return QFileInfo(baseDir + QLatin1Char('/') + path).absoluteFilePath();
+        return QFileInfo(baseDir + '/' + path).absoluteFilePath();
     return path;
 }
 
 void PathChooser::setPath(const QString &path)
 {
-    d->m_lineEdit->setText(QDir::toNativeSeparators(path));
+    d->m_lineEdit->setTextKeepingActiveCursor(QDir::toNativeSeparators(path));
 }
 
-void PathChooser::setFileName(const FileName &fn)
+void PathChooser::setFileName(const FilePath &fn)
 {
-    d->m_lineEdit->setText(fn.toUserOutput());
-}
-
-void PathChooser::setErrorColor(const QColor &errorColor)
-{
-    d->m_lineEdit->setErrorColor(errorColor);
-}
-
-void PathChooser::setOkColor(const QColor &okColor)
-{
-    d->m_lineEdit->setOkColor(okColor);
+    d->m_lineEdit->setTextKeepingActiveCursor(fn.toUserOutput());
 }
 
 bool PathChooser::isReadOnly() const
@@ -370,7 +355,8 @@ bool PathChooser::isReadOnly() const
 void PathChooser::setReadOnly(bool b)
 {
     d->m_lineEdit->setReadOnly(b);
-    foreach (QAbstractButton *button, d->m_buttons)
+    const auto buttons = d->m_buttons;
+    for (QAbstractButton *button : buttons)
         button->setEnabled(!b);
 }
 
@@ -443,6 +429,10 @@ void PathChooser::slotBrowse()
         break;
     }
 
+    // work around QTBUG-61004 / QTCREATORBUG-22906
+    window()->raise();
+    window()->activateWindow();
+
     // Delete trailing slashes unless it is "/"|"\\", only
     if (!newPath.isEmpty()) {
         newPath = QDir::toNativeSeparators(newPath);
@@ -487,16 +477,6 @@ void PathChooser::setAboutToShowContextMenuHandler(PathChooser::AboutToShowConte
     s_aboutToShowContextMenuHandler = handler;
 }
 
-QColor PathChooser::errorColor() const
-{
-    return d->m_lineEdit->errorColor();
-}
-
-QColor PathChooser::okColor() const
-{
-    return d->m_lineEdit->okColor();
-}
-
 FancyLineEdit::ValidationFunction PathChooser::defaultValidationFunction() const
 {
     return std::bind(&PathChooser::validatePath, this, std::placeholders::_1, std::placeholders::_2);
@@ -522,7 +502,7 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
 
     // Check if existing
     switch (d->m_acceptingKind) {
-    case PathChooser::ExistingDirectory: // fall through
+    case PathChooser::ExistingDirectory:
         if (!fi.exists()) {
             if (errorMessage)
                 *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
@@ -534,7 +514,7 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
             return false;
         }
         break;
-    case PathChooser::File: // fall through
+    case PathChooser::File:
         if (!fi.exists()) {
             if (errorMessage)
                 *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
@@ -577,7 +557,7 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
             return false;
         }
         break;
-    case PathChooser::Command: // fall through
+    case PathChooser::Command:
         if (fi.exists() && !fi.isExecutable()) {
             if (errorMessage)
                 *errorMessage = tr("Cannot execute \"%1\".").arg(QDir::toNativeSeparators(expandedPath));
@@ -640,6 +620,7 @@ QString PathChooser::promptDialogTitle() const
 void PathChooser::setPromptDialogFilter(const QString &filter)
 {
     d->m_dialogFilter = filter;
+    d->m_lineEdit->validate();
 }
 
 QString PathChooser::promptDialogFilter() const
@@ -664,18 +645,18 @@ FancyLineEdit *PathChooser::lineEdit() const
 {
     // HACK: Make it work with HistoryCompleter.
     if (d->m_lineEdit->objectName().isEmpty())
-        d->m_lineEdit->setObjectName(objectName() + QLatin1String("LineEdit"));
+        d->m_lineEdit->setObjectName(objectName() + "LineEdit");
     return d->m_lineEdit;
 }
 
 QString PathChooser::toolVersion(const QString &binary, const QStringList &arguments)
 {
-    return BinaryVersionToolTipEventFilter::toolVersion(binary, arguments);
+    return BinaryVersionToolTipEventFilter::toolVersion({binary, arguments});
 }
 
 void PathChooser::installLineEditVersionToolTip(QLineEdit *le, const QStringList &arguments)
 {
-    BinaryVersionToolTipEventFilter *ef = new BinaryVersionToolTipEventFilter(le);
+    auto ef = new BinaryVersionToolTipEventFilter(le);
     ef->setArguments(arguments);
 }
 
@@ -684,6 +665,11 @@ void PathChooser::setHistoryCompleter(const QString &historyKey, bool restoreLas
     d->m_lineEdit->setHistoryCompleter(historyKey, restoreLastItemFromHistory);
 }
 
+void PathChooser::setMacroExpander(MacroExpander *macroExpander)
+{
+    d->m_macroExpander = macroExpander;
+}
+
 QStringList PathChooser::commandVersionArguments() const
 {
     return d->m_binaryVersionToolTipEventFilter ?
@@ -696,7 +682,7 @@ void PathChooser::setCommandVersionArguments(const QStringList &arguments)
     if (arguments.isEmpty()) {
         if (d->m_binaryVersionToolTipEventFilter) {
             delete d->m_binaryVersionToolTipEventFilter;
-            d->m_binaryVersionToolTipEventFilter = 0;
+            d->m_binaryVersionToolTipEventFilter = nullptr;
         }
     } else {
         if (!d->m_binaryVersionToolTipEventFilter)
