@@ -27,25 +27,28 @@
 #include "locatorfilter.h"
 #include "qtcreatorsymbolsfindfilter.h"
 #include "qtcreatoreditormanager.h"
+#include "qtcreatorrefactoringprojectupdater.h"
 #include "querysqlitestatementfactory.h"
 #include "sqlitedatabase.h"
 #include "sqlitereadstatement.h"
 #include "symbolquery.h"
 
-#include <clangpchmanager/qtcreatorprojectupdater.h>
+#include <clangpchmanager/clangpchmanagerplugin.h>
+#include <clangpchmanager/progressmanager.h>
 #include <clangsupport/refactoringdatabaseinitializer.h>
+#include <projectpartsstorage.h>
 
 #include <cpptools/cppmodelmanager.h>
 
 #include <coreplugin/icore.h>
-#include <extensionsystem/pluginmanager.h>
+#include <coreplugin/progressmanager/progressmanager.h>
 #include <cpptools/cpptoolsconstants.h>
+#include <extensionsystem/pluginmanager.h>
 
 #include <refactoringdatabaseinitializer.h>
 #include <filepathcaching.h>
 
 #include <sqlitedatabase.h>
-
 #include <utils/hostosinfo.h>
 
 #include <QDir>
@@ -59,6 +62,8 @@ namespace ClangRefactoring {
 
 namespace {
 
+using ClangPchManager::ClangPchManagerPlugin;
+
 QString backendProcessPath()
 {
     return Core::ICore::libexecPath()
@@ -72,34 +77,35 @@ std::unique_ptr<ClangRefactoringPluginData> ClangRefactoringPlugin::d;
 
 class ClangRefactoringPluginData
 {
-    using ProjectUpdater = ClangPchManager::QtCreatorProjectUpdater<ClangPchManager::ProjectUpdater>;
 public:
     using QuerySqliteReadStatementFactory = QuerySqliteStatementFactory<Sqlite::Database,
                                                                         Sqlite::ReadStatement>;
-    Sqlite::Database database{Utils::PathString{Core::ICore::userResourcePath() + "/symbol-experimental-v1.db"}, 1000ms};
+    Sqlite::Database database{Utils::PathString{Core::ICore::cacheResourcePath()
+                                                + "/symbol-experimental-v1.db"},
+                              1000ms};
     ClangBackEnd::RefactoringDatabaseInitializer<Sqlite::Database> databaseInitializer{database};
     ClangBackEnd::FilePathCaching filePathCache{database};
-    RefactoringClient refactoringClient;
+    ClangPchManager::ProgressManager progressManager{
+        [] (QFutureInterface<void> &promise) {
+            auto title = QCoreApplication::translate("ClangRefactoringProgressManager", "C++ Indexing");
+            Core::ProgressManager::addTask(promise.future(), title, "clang indexing", {});}};
+    RefactoringClient refactoringClient{progressManager};
     QtCreatorEditorManager editorManager{filePathCache};
     ClangBackEnd::RefactoringConnectionClient connectionClient{&refactoringClient};
     QuerySqliteReadStatementFactory statementFactory{database};
     SymbolQuery<QuerySqliteReadStatementFactory> symbolQuery{statementFactory};
+    ClangBackEnd::ProjectPartsStorage<Sqlite::Database> projectPartsStorage{database};
     RefactoringEngine engine{connectionClient.serverProxy(), refactoringClient, filePathCache, symbolQuery};
-
-    QtCreatorSearch qtCreatorSearch;
-    QtCreatorClangQueryFindFilter qtCreatorfindFilter{connectionClient.serverProxy(),
-                                                      qtCreatorSearch,
-                                                      refactoringClient};
-    ProjectUpdater projectUpdate{connectionClient.serverProxy(), filePathCache};
+    QtCreatorRefactoringProjectUpdater projectUpdate{connectionClient.serverProxy(),
+                                                     ClangPchManagerPlugin::pchManagerClient(),
+                                                     filePathCache,
+                                                     projectPartsStorage,
+                                                     ClangPchManagerPlugin::settingsManager()};
 };
 
-ClangRefactoringPlugin::ClangRefactoringPlugin()
-{
-}
+ClangRefactoringPlugin::ClangRefactoringPlugin() = default;
 
-ClangRefactoringPlugin::~ClangRefactoringPlugin()
-{
-}
+ClangRefactoringPlugin::~ClangRefactoringPlugin() = default;
 
 static bool useClangFilters()
 {
@@ -113,26 +119,20 @@ bool ClangRefactoringPlugin::initialize(const QStringList & /*arguments*/, QStri
 
     d->refactoringClient.setRefactoringEngine(&d->engine);
     d->refactoringClient.setRefactoringConnectionClient(&d->connectionClient);
-    ExtensionSystem::PluginManager::addObject(&d->qtCreatorfindFilter);
 
     connectBackend();
     startBackend();
 
-    CppTools::CppModelManager::addRefactoringEngine(
-                CppTools::RefactoringEngineType::ClangRefactoring, &refactoringEngine());
+    CppTools::CppModelManager::addRefactoringEngine(CppTools::RefactoringEngineType::ClangRefactoring,
+                                                    &refactoringEngine());
 
     initializeFilters();
 
     return true;
 }
 
-void ClangRefactoringPlugin::extensionsInitialized()
-{
-}
-
 ExtensionSystem::IPlugin::ShutdownFlag ClangRefactoringPlugin::aboutToShutdown()
 {
-    ExtensionSystem::PluginManager::removeObject(&d->qtCreatorfindFilter);
     CppTools::CppModelManager::removeRefactoringEngine(
                 CppTools::RefactoringEngineType::ClangRefactoring);
     d->refactoringClient.setRefactoringConnectionClient(nullptr);
