@@ -33,6 +33,7 @@
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
+#include <utils/utilsicons.h>
 
 #include <QDir>
 
@@ -146,6 +147,7 @@ PreprocessedData preprocess(FileApiData &data,
 
     // Simplify to only one configuration:
     result.codemodel = extractConfiguration(data.codemodel, errorMessage);
+
     if (!errorMessage.isEmpty()) {
         return result;
     }
@@ -207,6 +209,8 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
 
     const QList<CMakeBuildTarget> result = transform<QList>(
         input.targetDetails, [&sourceDir, &buildDir](const TargetDetails &t) -> CMakeBuildTarget {
+            const auto currentBuildDir = QDir(buildDir.absoluteFilePath(t.buildDir.toString()));
+
             CMakeBuildTarget ct;
             ct.title = t.name;
             ct.executable = t.artifacts.isEmpty()
@@ -225,9 +229,9 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
             else
                 type = UtilityType;
             ct.targetType = type;
-            ct.workingDirectory = ct.executable.isEmpty() ? FilePath::fromString(
-                                      buildDir.absoluteFilePath(t.buildDir.toString()))
-                                                          : ct.executable.parentDir();
+            ct.workingDirectory = ct.executable.isEmpty()
+                                      ? FilePath::fromString(currentBuildDir.absolutePath())
+                                      : ct.executable.parentDir();
             ct.sourceDirectory = FilePath::fromString(
                 QDir::cleanPath(sourceDir.absoluteFilePath(t.sourceDir.toString())));
 
@@ -256,6 +260,40 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
                     extractBacktraceInformation(t.backtraceGraph, sourceDir, id.backtrace, 500));
             }
 
+            // Is this a terminal application?
+            Utils::FilePaths librarySeachPaths;
+            if (ct.targetType == ExecutableType && t.link && t.link.value().language == "CXX") {
+                for (const FragmentInfo &f : t.link.value().fragments) {
+                    FilePath tmp;
+                    // Some projects abuse linking to libraries to pass random flags to the linker!
+                    if (f.role != "flags"
+                        && !(f.fragment.startsWith("-") || f.fragment.contains(" -"))) {
+                        tmp = FilePath::fromString(currentBuildDir.absoluteFilePath(
+                            QDir::fromNativeSeparators(f.fragment)));
+                    }
+
+                    if (f.role == "libraries") {
+                        tmp = tmp.parentDir();
+
+                        if (f.fragment.contains("QtGui") || f.fragment.contains("Qt5Gui")
+                            || f.fragment.contains("Qt6Gui"))
+                            ct.linksToQtGui = true;
+                    }
+
+                    if (!tmp.isEmpty()) {
+                        librarySeachPaths.append(tmp);
+                        // Libraries often have their import libs in ../lib and the
+                        // actual dll files in ../bin on windows. Qt is one example of that.
+                        if (tmp.fileName() == "lib" && HostOsInfo::isWindowsHost()) {
+                            const FilePath path = tmp.parentDir().pathAppended("bin");
+
+                            librarySeachPaths.append(path);
+                        }
+                    }
+                }
+            }
+            ct.libraryDirectories = filteredUnique(librarySeachPaths);
+
             return ct;
         });
     return result;
@@ -271,15 +309,13 @@ static QStringList splitFragments(const QStringList &fragments)
 }
 
 RawProjectParts generateRawProjectParts(const PreprocessedData &input,
-                                        const FilePath &sourceDirectory,
-                                        const FilePath &buildDirectory)
+                                        const FilePath &sourceDirectory)
 {
     RawProjectParts rpps;
 
     int counter = 0;
     for (const TargetDetails &t : input.targetDetails) {
         QDir sourceDir(sourceDirectory.toString());
-        QDir buildDir(buildDirectory.toString());
 
         bool needPostfix = t.compileGroups.size() > 1;
         int count = 1;
@@ -333,11 +369,7 @@ RawProjectParts generateRawProjectParts(const PreprocessedData &input,
             }));
             if (!precompiled_header.isEmpty()) {
                 if (precompiled_header.toFileInfo().isRelative()) {
-                    const FilePath parentDir = FilePath::fromString(buildDir.absolutePath());
-                    const QString dirName = buildDir.dirName();
-                    if (precompiled_header.startsWith(dirName))
-                        precompiled_header = FilePath::fromString(
-                            precompiled_header.toString().mid(dirName.length() + 1));
+                    const FilePath parentDir = FilePath::fromString(sourceDir.absolutePath());
                     precompiled_header = parentDir.pathAppended(precompiled_header.toString());
                 }
                 rpp.setPreCompiledHeaders({precompiled_header.toString()});
@@ -384,31 +416,35 @@ void addProjects(const QHash<Utils::FilePath, ProjectNode *> &cmakeListsNodes,
     }
 }
 
-QVector<FolderNode *> addSourceGroups(ProjectNode *targetRoot,
-                                      const TargetDetails &td,
-                                      const FilePath &sourceDirectory)
+FolderNode *createSourceGroupNode(const QString &sourceGroupName,
+                                  const FilePath &sourceDirectory,
+                                  FolderNode *targetRoot)
 {
-    QVector<FolderNode *> sourceGroupNodes;
-    if (td.sourceGroups.size() == 1) {
-        sourceGroupNodes.append(
-            targetRoot); // Only one source group, so do not bother to display any:-)
-    } else {
-        for (const QString &sg : td.sourceGroups) {
-            if (sg.isEmpty()) {
-                sourceGroupNodes.append(targetRoot);
-            } else {
-                auto sgNode = createCMakeVFolder(sourceDirectory,
-                                                 Node::DefaultFolderPriority + 5,
-                                                 sg);
-                sgNode->setListInProject(false);
+    FolderNode *currentNode = targetRoot;
 
-                sourceGroupNodes.append(sgNode.get());
-                targetRoot->addNode(std::move(sgNode));
+    if (!sourceGroupName.isEmpty()) {
+        const QStringList parts = sourceGroupName.split("\\");
+
+        for (const QString &p : parts) {
+            FolderNode *existingNode = Utils::findOrDefault(currentNode->folderNodes(),
+                                                            [&p](const FolderNode *fn) {
+                                                                return fn->displayName() == p;
+                                                            });
+
+            if (!existingNode) {
+                auto node = createCMakeVFolder(sourceDirectory, Node::DefaultFolderPriority + 5, p);
+                node->setListInProject(false);
+                node->setIcon(QIcon::fromTheme("edit-copy", ::Utils::Icons::COPY.icon()));
+
+                existingNode = node.get();
+
+                currentNode->addNode(std::move(node));
             }
+
+            currentNode = existingNode;
         }
     }
-
-    return sourceGroupNodes;
+    return currentNode;
 }
 
 void addCompileGroups(ProjectNode *targetRoot,
@@ -428,11 +464,11 @@ void addCompileGroups(ProjectNode *targetRoot,
     targetRoot->forEachGenericNode(
         [&alreadyListed](const Node *n) { alreadyListed.insert(n->filePath()); });
 
-    QVector<FolderNode *> sourceGroupNodes = addSourceGroups(targetRoot, td, sourceDirectory);
     const QDir topSourceDir(topSourceDirectory.toString());
 
     std::vector<std::unique_ptr<FileNode>> buildFileNodes;
     std::vector<std::unique_ptr<FileNode>> otherFileNodes;
+    std::vector<std::vector<std::unique_ptr<FileNode>>> sourceGroupFileNodes{td.sourceGroups.size()};
 
     for (const SourceInfo &si : td.sources) {
         const FilePath sourcePath = FilePath::fromString(
@@ -456,12 +492,31 @@ void addCompileGroups(ProjectNode *targetRoot,
         if (sourcePath.isChildOf(buildDirectory) && !inSourceBuild) {
             buildFileNodes.emplace_back(std::move(node));
         } else if (sourcePath.isChildOf(sourceDirectory)) {
-            sourceGroupNodes[si.sourceGroup]->addNestedNode(std::move(node));
+            sourceGroupFileNodes[si.sourceGroup].emplace_back(std::move(node));
         } else {
             otherFileNodes.emplace_back(std::move(node));
         }
     }
 
+    // Calculate base directory for source groups:
+    for (size_t i = 0; i < sourceGroupFileNodes.size(); ++i) {
+        std::vector<std::unique_ptr<FileNode>> &current = sourceGroupFileNodes[i];
+        FilePath baseDirectory;
+        // All the sourceGroupFileNodes are below sourceDirectory, so this is safe:
+        for (const std::unique_ptr<FileNode> &fn : current) {
+            if (baseDirectory.isEmpty()) {
+                baseDirectory = fn->filePath().parentDir();
+            } else {
+                baseDirectory = Utils::FileUtils::commonPath(baseDirectory, fn->filePath());
+            }
+        }
+
+        FolderNode *insertNode = createSourceGroupNode(td.sourceGroups[i],
+                                                       baseDirectory,
+                                                       targetRoot);
+        insertNode->addNestedNodes(std::move(sourceGroupFileNodes[i]));
+    }
+
     addCMakeVFolder(targetRoot,
                     buildDirectory,
                     100,
@@ -613,7 +668,7 @@ FileApiQtcData extractData(FileApiData &input,
 
     result.buildTargets = generateBuildTargets(data, sourceDirectory, buildDirectory);
     result.cmakeFiles = std::move(data.cmakeFiles);
-    result.projectParts = generateRawProjectParts(data, sourceDirectory, buildDirectory);
+    result.projectParts = generateRawProjectParts(data, sourceDirectory);
 
     auto pair = generateRootProjectNode(data, sourceDirectory, buildDirectory);
     result.rootProjectNode = std::move(pair.first);
