@@ -36,7 +36,6 @@
 
 QT_BEGIN_NAMESPACE
 class QGroupBox;
-class QKeyEvent;
 class QLabel;
 QT_END_NAMESPACE
 
@@ -47,7 +46,7 @@ class Command;
 namespace Internal {
 
 class ActionManagerPrivate;
-class MainWindow;
+class ShortcutSettingsWidget;
 
 struct ShortcutItem
 {
@@ -60,15 +59,15 @@ class ShortcutButton : public QPushButton
 {
     Q_OBJECT
 public:
-    ShortcutButton(QWidget *parent = 0);
+    ShortcutButton(QWidget *parent = nullptr);
 
-    QSize sizeHint() const;
+    QSize sizeHint() const override;
 
 signals:
     void keySequenceChanged(const QKeySequence &sequence);
 
 protected:
-    bool eventFilter(QObject *obj, QEvent *evt);
+    bool eventFilter(QObject *obj, QEvent *evt) override;
 
 private:
     void updateText();
@@ -81,44 +80,10 @@ private:
     int m_keyNum = 0;
 };
 
-class ShortcutSettingsWidget : public CommandMappings
+class ShortcutSettings final : public IOptionsPage
 {
-    Q_OBJECT
-
 public:
-    ShortcutSettingsWidget(QWidget *parent = 0);
-    ~ShortcutSettingsWidget() override;
-
-    void apply();
-
-protected:
-    void importAction() override;
-    void exportAction() override;
-    void defaultAction() override;
-    bool filterColumn(const QString &filterString, QTreeWidgetItem *item, int column) const override;
-
-private:
-    void initialize();
-    void handleCurrentCommandChanged(QTreeWidgetItem *current);
-    void resetToDefault();
-    bool validateShortcutEdit() const;
-    bool markCollisions(ShortcutItem *);
-    void setKeySequence(const QKeySequence &key);
-    void showConflicts();
-    void clear();
-
-    QList<ShortcutItem *> m_scitems;
-    QGroupBox *m_shortcutBox;
-    Utils::FancyLineEdit *m_shortcutEdit;
-    QLabel *m_warningLabel;
-};
-
-class ShortcutSettings : public IOptionsPage
-{
-    Q_OBJECT
-
-public:
-    ShortcutSettings(QObject *parent = 0);
+    ShortcutSettings();
 
     QWidget *widget() override;
     void apply() override;
