@@ -30,7 +30,12 @@
 #include <QWidget>
 #include <QTextCharFormat>
 
-namespace TextEditor { class FontSettings; }
+namespace Core { class IContext; }
+
+namespace TextEditor {
+class FontSettings;
+class TextEditorWidget;
+}
 
 QT_BEGIN_NAMESPACE
 class QMenu;
@@ -39,7 +44,6 @@ QT_END_NAMESPACE
 
 namespace DiffEditor {
 
-class DiffEditorController;
 class FileData;
 
 namespace Internal {
@@ -51,7 +55,11 @@ class SideBySideDiffEditorWidget : public QWidget
 {
     Q_OBJECT
 public:
-    explicit SideBySideDiffEditorWidget(QWidget *parent = 0);
+    explicit SideBySideDiffEditorWidget(QWidget *parent = nullptr);
+    ~SideBySideDiffEditorWidget() override;
+
+    TextEditor::TextEditorWidget *leftEditorWidget() const;
+    TextEditor::TextEditorWidget *rightEditorWidget() const;
 
     void setDocument(DiffEditorDocument *document);
     DiffEditorDocument *diffDocument() const;
@@ -76,28 +84,33 @@ private:
                                              int lineNumber, int columnNumber);
     void slotRightJumpToOriginalFileRequested(int diffFileIndex,
                                               int lineNumber, int columnNumber);
-    void slotLeftContextMenuRequested(QMenu *menu, int diffFileIndex,
-                                      int chunkIndex);
-    void slotRightContextMenuRequested(QMenu *menu, int diffFileIndex,
-                                       int chunkIndex);
+    void slotLeftContextMenuRequested(QMenu *menu, int fileIndex,
+                                      int chunkIndex, const ChunkSelection &selection);
+    void slotRightContextMenuRequested(QMenu *menu, int fileIndex,
+                                       int chunkIndex, const ChunkSelection &selection);
     void leftVSliderChanged();
     void rightVSliderChanged();
     void leftHSliderChanged();
     void rightHSliderChanged();
     void leftCursorPositionChanged();
     void rightCursorPositionChanged();
+    void syncHorizontalScrollBarPolicy();
+    void handlePositionChange(SideDiffEditorWidget *source, SideDiffEditorWidget *dest);
+    void syncCursor(SideDiffEditorWidget *source, SideDiffEditorWidget *dest);
 
     void showDiff();
 
-    SideDiffEditorWidget *m_leftEditor;
-    SideDiffEditorWidget *m_rightEditor;
-    QSplitter *m_splitter;
+    SideDiffEditorWidget *m_leftEditor = nullptr;
+    SideDiffEditorWidget *m_rightEditor = nullptr;
+    QSplitter *m_splitter = nullptr;
 
     DiffEditorWidgetController m_controller;
 
     bool m_horizontalSync = false;
 
     QTextCharFormat m_spanLineFormat;
+    Core::IContext *m_leftContext = nullptr;
+    Core::IContext *m_rightContext = nullptr;
 };
 
 } // namespace Internal
