@@ -1,7 +1,7 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (C) 2015 Brian McGillion
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 Brian McGillion
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,35 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "mercurialclient.h"
+#include "mercurialplugin.h"
 #include "constants.h"
 
+#include <coreplugin/idocument.h>
+
 #include <vcsbase/vcscommand.h>
 #include <vcsbase/vcsoutputwindow.h>
 #include <vcsbase/vcsbaseplugin.h>
 #include <vcsbase/vcsbaseeditor.h>
-#include <vcsbase/vcsbaseeditorparameterwidget.h>
+#include <vcsbase/vcsbaseeditorconfig.h>
+#include <vcsbase/vcsbasediffeditorcontroller.h>
 #include <utils/synchronousprocess.h>
 #include <utils/fileutils.h>
+#include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 
 #include <QDateTime>
@@ -47,20 +47,47 @@
 #include <QTextStream>
 #include <QVariant>
 
+using namespace Core;
+using namespace DiffEditor;
 using namespace Utils;
 using namespace VcsBase;
 
 namespace Mercurial {
 namespace Internal  {
 
-MercurialClient::MercurialClient(MercurialSettings *settings) :
-    VcsBaseClient(settings)
+class MercurialDiffEditorController : public VcsBaseDiffEditorController
 {
+public:
+    MercurialDiffEditorController(IDocument *document)
+        : VcsBaseDiffEditorController(document)
+    {
+        setDisplayName("Hg Diff");
+    }
+
+    void runCommand(const QList<QStringList> &args, QTextCodec *codec = nullptr);
+    QStringList addConfigurationArguments(const QStringList &args) const;
+};
+
+void MercurialDiffEditorController::runCommand(const QList<QStringList> &args, QTextCodec *codec)
+{
+    // at this moment, don't ignore any errors
+    VcsBaseDiffEditorController::runCommand(args, 0u, codec);
 }
 
-MercurialSettings *MercurialClient::settings() const
+QStringList MercurialDiffEditorController::addConfigurationArguments(const QStringList &args) const
+{
+    QStringList configArgs{ "-g", "-p" };
+    configArgs << "-U" << QString::number(contextLineCount());
+    if (ignoreWhitespace()) {
+        configArgs << "-w" << "-b" << "-B" << "-Z";
+    }
+    return args + configArgs;
+}
+
+/////////////////////////////////////////////////////////////
+
+MercurialClient::MercurialClient(MercurialSettings *settings) : VcsBaseClient(settings)
 {
-    return dynamic_cast<MercurialSettings *>(VcsBaseClient::settings());
 }
 
 bool MercurialClient::manifestSync(const QString &repository, const QString &relativeFilename)
@@ -68,12 +95,12 @@ bool MercurialClient::manifestSync(const QString &repository, const QString &rel
     // This  only works when called from the repo and outputs paths relative to it.
     const QStringList args(QLatin1String("manifest"));
 
-    QByteArray output;
-    vcsFullySynchronousExec(repository, args, &output);
+    const SynchronousProcessResponse result = vcsFullySynchronousExec(repository, args);
+
     const QDir repositoryDir(repository);
     const QFileInfo needle = QFileInfo(repositoryDir, relativeFilename);
 
-    const QStringList files = QString::fromLocal8Bit(output).split(QLatin1Char('\n'));
+    const QStringList files = result.stdOut().split(QLatin1Char('\n'));
     foreach (const QString &fileName, files) {
         const QFileInfo managedFile(repositoryDir, fileName);
         if (needle == managedFile)
@@ -88,25 +115,26 @@ bool MercurialClient::synchronousClone(const QString &workingDir,
                                        const QString &dstLocation,
                                        const QStringList &extraOptions)
 {
-    Q_UNUSED(workingDir);
-    Q_UNUSED(extraOptions);
+    Q_UNUSED(workingDir)
+    Q_UNUSED(extraOptions)
     QDir workingDirectory(srcLocation);
-    QByteArray output;
-    const unsigned flags = VcsBasePlugin::SshPasswordPrompt |
-            VcsBasePlugin::ShowStdOutInLogWindow |
-            VcsBasePlugin::ShowSuccessMessage;
+    const unsigned flags = VcsCommand::SshPasswordPrompt |
+            VcsCommand::ShowStdOut |
+            VcsCommand::ShowSuccessMessage;
 
     if (workingDirectory.exists()) {
         // Let's make first init
         QStringList arguments(QLatin1String("init"));
-        if (!vcsFullySynchronousExec(workingDirectory.path(), arguments, &output))
+        const SynchronousProcessResponse resp = vcsFullySynchronousExec(
+                    workingDirectory.path(), arguments);
+        if (resp.result != SynchronousProcessResponse::Finished)
             return false;
 
         // Then pull remote repository
         arguments.clear();
         arguments << QLatin1String("pull") << dstLocation;
-        const SynchronousProcessResponse resp1 =
-                vcsSynchronousExec(workingDirectory.path(), arguments, flags);
+        const SynchronousProcessResponse resp1 = vcsSynchronousExec(
+                    workingDirectory.path(), arguments, flags);
         if (resp1.result != SynchronousProcessResponse::Finished)
             return false;
 
@@ -122,15 +150,15 @@ bool MercurialClient::synchronousClone(const QString &workingDir,
         // And last update repository
         arguments.clear();
         arguments << QLatin1String("update");
-        const SynchronousProcessResponse resp2 =
-                vcsSynchronousExec(workingDirectory.path(), arguments, flags);
+        const SynchronousProcessResponse resp2 = vcsSynchronousExec(
+                    workingDirectory.path(), arguments, flags);
         return resp2.result == SynchronousProcessResponse::Finished;
     } else {
         QStringList arguments(QLatin1String("clone"));
         arguments << dstLocation << workingDirectory.dirName();
         workingDirectory.cdUp();
-        const SynchronousProcessResponse resp =
-                vcsSynchronousExec(workingDirectory.path(), arguments, flags);
+        const SynchronousProcessResponse resp = vcsSynchronousExec(
+                    workingDirectory.path(), arguments, flags);
         return resp.result == SynchronousProcessResponse::Finished;
     }
 }
@@ -141,20 +169,18 @@ bool MercurialClient::synchronousPull(const QString &workingDir, const QString &
     args << vcsCommandString(PullCommand) << extraOptions << srcLocation;
     // Disable UNIX terminals to suppress SSH prompting
     const unsigned flags =
-            VcsBasePlugin::SshPasswordPrompt
-            | VcsBasePlugin::ShowStdOutInLogWindow
-            | VcsBasePlugin::ShowSuccessMessage;
-    const FileName binary = settings()->binaryPath();
-    const int timeoutSec = settings()->value(settings()->timeoutKey).toInt();
+            VcsCommand::SshPasswordPrompt
+            | VcsCommand::ShowStdOut
+            | VcsCommand::ShowSuccessMessage;
 
     // cause mercurial doesn`t understand LANG
     QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
     env.insert(QLatin1String("LANGUAGE"), QLatin1String("C"));
-    const SynchronousProcessResponse resp = VcsBasePlugin::runVcs(
-                workingDir, binary, args, timeoutSec * 1000, flags, 0, env);
+    const SynchronousProcessResponse resp = VcsBase::runVcs(
+                workingDir, {vcsBinary(), args}, vcsTimeoutS(), flags, nullptr, env);
     const bool ok = resp.result == SynchronousProcessResponse::Finished;
 
-    parsePullOutput(resp.stdOut.trimmed());
+    parsePullOutput(resp.stdOut().trimmed());
     return ok;
 }
 
@@ -191,24 +217,22 @@ QStringList MercurialClient::parentRevisionsSync(const QString &workingDirectory
     args << QLatin1String("parents") <<  QLatin1String("-r") <<revision;
     if (!file.isEmpty())
         args << file;
-    QByteArray outputData;
-    if (!vcsFullySynchronousExec(workingDirectory, args, &outputData))
+    const SynchronousProcessResponse resp = vcsFullySynchronousExec(workingDirectory, args);
+    if (resp.result != SynchronousProcessResponse::Finished)
         return QStringList();
-    const QString output = SynchronousProcess::normalizeNewlines(
-                QString::fromLocal8Bit(outputData));
     /* Looks like: \code
 changeset:   0:031a48610fba
 user: ...
 \endcode   */
     // Obtain first line and split by blank-delimited tokens
-    const QStringList lines = output.split(QLatin1Char('\n'));
+    const QStringList lines = resp.stdOut().split(QLatin1Char('\n'));
     if (lines.size() < 1) {
-        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
+        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(resp.stdOut())));
         return QStringList();
     }
     QStringList changeSets = lines.front().simplified().split(QLatin1Char(' '));
     if (changeSets.size() < 2) {
-        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
+        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(resp.stdOut())));
         return QStringList();
     }
     // Remove revision numbers
@@ -228,18 +252,15 @@ QString MercurialClient::shortDescriptionSync(const QString &workingDirectory,
                                            const QString &revision,
                                            const QString &format)
 {
-    QString description;
     QStringList args;
     args << QLatin1String("log") <<  QLatin1String("-r") <<revision;
     if (!format.isEmpty())
         args << QLatin1String("--template") << format;
-    QByteArray outputData;
-    if (!vcsFullySynchronousExec(workingDirectory, args, &outputData))
+
+    const SynchronousProcessResponse resp = vcsFullySynchronousExec(workingDirectory, args);
+    if (resp.result != SynchronousProcessResponse::Finished)
         return revision;
-    description = Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(outputData));
-    if (description.endsWith(QLatin1Char('\n')))
-        description.truncate(description.size() - 1);
-    return description;
+    return stripLastNewline(resp.stdOut());
 }
 
 // Default format: "SHA1 (author summmary)"
@@ -251,25 +272,11 @@ QString MercurialClient::shortDescriptionSync(const QString &workingDirectory,
     return shortDescriptionSync(workingDirectory, revision, QLatin1String(defaultFormatC));
 }
 
-QString MercurialClient::vcsGetRepositoryURL(const QString &directory)
-{
-    QByteArray output;
-
-    QStringList arguments(QLatin1String("showconfig"));
-    arguments << QLatin1String("paths.default");
-
-    if (vcsFullySynchronousExec(directory, arguments, &output))
-        return QString::fromLocal8Bit(output);
-    return QString();
-}
-
 bool MercurialClient::managesFile(const QString &workingDirectory, const QString &fileName) const
 {
     QStringList args;
     args << QLatin1String("status") << QLatin1String("--unknown") << fileName;
-    QByteArray output;
-    vcsFullySynchronousExec(workingDirectory, args, &output);
-    return output.isEmpty();
+    return vcsFullySynchronousExec(workingDirectory, args).stdOut().isEmpty();
 }
 
 void MercurialClient::incoming(const QString &repositoryRoot, const QString &repository)
@@ -280,15 +287,14 @@ void MercurialClient::incoming(const QString &repositoryRoot, const QString &rep
         args.append(repository);
 
     QString id = repositoryRoot;
-    if (!repository.isEmpty()) {
-        id += QDir::separator();
-        id += repository;
-    }
+    if (!repository.isEmpty())
+        id += QLatin1Char('/') + repository;
 
     const QString title = tr("Hg incoming %1").arg(id);
 
     VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG_ID, title, repositoryRoot,
-                                                  true, "incoming", id);
+                                                  VcsBaseEditor::getCodec(repositoryRoot),
+                                                  "incoming", id);
     VcsCommand *cmd = createCommand(repository, editor);
     enqueueJob(cmd, args);
 }
@@ -301,20 +307,21 @@ void MercurialClient::outgoing(const QString &repositoryRoot)
     const QString title = tr("Hg outgoing %1").
             arg(QDir::toNativeSeparators(repositoryRoot));
 
-    VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG_ID, title, repositoryRoot, true,
+    VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG_ID, title, repositoryRoot,
+                                                  VcsBaseEditor::getCodec(repositoryRoot),
                                                   "outgoing", repositoryRoot);
 
     VcsCommand *cmd = createCommand(repositoryRoot, editor);
     enqueueJob(cmd, args);
 }
 
-void MercurialClient::annotate(const QString &workingDir, const QString &file,
-                               const QString &revision, int lineNumber,
-                               const QStringList &extraOptions)
+VcsBaseEditorWidget *MercurialClient::annotate(
+        const QString &workingDir, const QString &file, const QString &revision,
+        int lineNumber, const QStringList &extraOptions)
 {
     QStringList args(extraOptions);
     args << QLatin1String("-u") << QLatin1String("-c") << QLatin1String("-d");
-    VcsBaseClient::annotate(workingDir, file, revision, lineNumber, args);
+    return VcsBaseClient::annotate(workingDir, file, revision, lineNumber, args);
 }
 
 void MercurialClient::commit(const QString &repositoryRoot, const QStringList &files,
@@ -329,9 +336,30 @@ void MercurialClient::commit(const QString &repositoryRoot, const QStringList &f
 void MercurialClient::diff(const QString &workingDir, const QStringList &files,
                            const QStringList &extraOptions)
 {
-    QStringList args(extraOptions);
-    args << QLatin1String("-g") << QLatin1String("-p") << QLatin1String("-U 8");
-    VcsBaseClient::diff(workingDir, files, args);
+    Q_UNUSED(extraOptions)
+
+    QString fileName;
+
+    if (files.empty()) {
+        const QString title = tr("Mercurial Diff");
+        const QString sourceFile = VcsBaseEditor::getSource(workingDir, fileName);
+        const QString documentId = QString(Constants::MERCURIAL_PLUGIN)
+                + ".DiffRepo." + sourceFile;
+        requestReload(documentId, sourceFile, title, workingDir, {"diff"});
+    } else if (files.size() == 1) {
+        fileName = files.at(0);
+        const QString title = tr("Mercurial Diff \"%1\"").arg(fileName);
+        const QString sourceFile = VcsBaseEditor::getSource(workingDir, fileName);
+        const QString documentId = QString(Constants::MERCURIAL_PLUGIN)
+                + ".DiffFile." + sourceFile;
+        requestReload(documentId, sourceFile, title, workingDir, {"diff", fileName});
+    } else {
+        const QString title = tr("Mercurial Diff \"%1\"").arg(workingDir);
+        const QString sourceFile = VcsBaseEditor::getSource(workingDir, fileName);
+        const QString documentId = QString(Constants::MERCURIAL_PLUGIN)
+                + ".DiffFile." + workingDir;
+        requestReload(documentId, sourceFile, title, workingDir, QStringList{"diff"} + files);
+    }
 }
 
 void MercurialClient::import(const QString &repositoryRoot, const QStringList &files,
@@ -348,6 +376,12 @@ void MercurialClient::revertAll(const QString &workingDir, const QString &revisi
                              QStringList(extraOptions) << QLatin1String("--all"));
 }
 
+bool MercurialClient::isVcsDirectory(const FilePath &fileName) const
+{
+    return fileName.isDir()
+            && !fileName.fileName().compare(Constants::MERCURIALREPO, HostOsInfo::fileNameCaseSensitivity());
+}
+
 void MercurialClient::view(const QString &source, const QString &id,
                            const QStringList &extraOptions)
 {
@@ -361,8 +395,8 @@ QString MercurialClient::findTopLevelForFile(const QFileInfo &file) const
 {
     const QString repositoryCheckFile = QLatin1String(Constants::MERCURIALREPO) + QLatin1String("/requires");
     return file.isDir() ?
-                VcsBasePlugin::findRepositoryForDirectory(file.absoluteFilePath(), repositoryCheckFile) :
-                VcsBasePlugin::findRepositoryForDirectory(file.absolutePath(), repositoryCheckFile);
+                VcsBase::findRepositoryForDirectory(file.absoluteFilePath(), repositoryCheckFile) :
+                VcsBase::findRepositoryForDirectory(file.absolutePath(), repositoryCheckFile);
 }
 
 Core::Id MercurialClient::vcsEditorKind(VcsCommandTag cmd) const
@@ -407,11 +441,33 @@ MercurialClient::StatusItem MercurialClient::parseStatusLine(const QString &line
 
         //the status line should be similar to "M file_with_changes"
         //so just should take the file name part and store it
-        item.file = line.mid(2);
+        item.file = QDir::fromNativeSeparators(line.mid(2));
     }
     return item;
 }
 
+void MercurialClient::requestReload(const QString &documentId, const QString &source, const QString &title,
+                                    const QString &workingDirectory, const QStringList &args)
+{
+    // Creating document might change the referenced source. Store a copy and use it.
+    const QString sourceCopy = source;
+
+    IDocument *document = DiffEditorController::findOrCreateDocument(documentId, title);
+    QTC_ASSERT(document, return);
+    auto controller = new MercurialDiffEditorController(document);
+    controller->setReloader([controller, args] {
+        controller->runCommand({controller->addConfigurationArguments(args)});
+    });
+    controller->setVcsBinary(settings().binaryPath());
+    controller->setVcsTimeoutS(settings().vcsTimeoutS());
+    controller->setProcessEnvironment(processEnvironment());
+    controller->setWorkingDirectory(workingDirectory);
+
+    VcsBase::setSource(document, sourceCopy);
+    EditorManager::activateEditorForDocument(document);
+    controller->requestReload();
+}
+
 void MercurialClient::parsePullOutput(const QString &output)
 {
     if (output.endsWith(QLatin1String("no changes found")))
@@ -426,51 +482,5 @@ void MercurialClient::parsePullOutput(const QString &output)
         emit needMerge();
 }
 
-// Collect all parameters required for a diff to be able to associate them
-// with a diff editor and re-run the diff with parameters.
-struct MercurialDiffParameters
-{
-    QString workingDir;
-    QStringList files;
-    QStringList extraOptions;
-};
-
-// Parameter widget controlling whitespace diff mode, associated with a parameter
-class MercurialDiffParameterWidget : public VcsBaseEditorParameterWidget
-{
-    Q_OBJECT
-public:
-    MercurialDiffParameterWidget(MercurialClient *client,
-                                 const MercurialDiffParameters &p, QWidget *parent = 0) :
-        VcsBaseEditorParameterWidget(parent), m_client(client), m_params(p)
-    {
-        mapSetting(addToggleButton(QLatin1String("-w"), tr("Ignore Whitespace")),
-                   client->settings()->boolPointer(MercurialSettings::diffIgnoreWhiteSpaceKey));
-        mapSetting(addToggleButton(QLatin1String("-B"), tr("Ignore Blank Lines")),
-                   client->settings()->boolPointer(MercurialSettings::diffIgnoreBlankLinesKey));
-    }
-
-    void executeCommand()
-    {
-        m_client->diff(m_params.workingDir, m_params.files, m_params.extraOptions);
-    }
-
-private:
-    MercurialClient *m_client;
-    const MercurialDiffParameters m_params;
-};
-
-VcsBaseEditorParameterWidget *MercurialClient::createDiffEditor(
-    const QString &workingDir, const QStringList &files, const QStringList &extraOptions)
-{
-    MercurialDiffParameters parameters;
-    parameters.workingDir = workingDir;
-    parameters.files = files;
-    parameters.extraOptions = extraOptions;
-    return new MercurialDiffParameterWidget(this, parameters);
-}
-
 } // namespace Internal
 } // namespace Mercurial
-
-#include "mercurialclient.moc"
