@@ -32,67 +32,47 @@
 #include "projectexplorer.h"
 #include "target.h"
 
-#include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
 
-using namespace ProjectExplorer;
-
-namespace {
-
-IBuildStepFactory *findCloneFactory(BuildStepList *parent, BuildStep *source)
-{
-    return ExtensionSystem::PluginManager::getObject<IBuildStepFactory>(
-        [&parent, &source](IBuildStepFactory *factory) {
-            return factory->canClone(parent, source);
-        });
-}
-
-IBuildStepFactory *findRestoreFactory(BuildStepList *parent, const QVariantMap &map)
-{
-    return ExtensionSystem::PluginManager::getObject<IBuildStepFactory>(
-        [&parent, &map](IBuildStepFactory *factory) {
-            return factory->canRestore(parent, map);
-        });
-}
+namespace ProjectExplorer {
 
 const char STEPS_COUNT_KEY[] = "ProjectExplorer.BuildStepList.StepsCount";
 const char STEPS_PREFIX[] = "ProjectExplorer.BuildStepList.Step.";
 
-} // namespace
-
-BuildStepList::BuildStepList(QObject *parent, Core::Id id) :
-    ProjectConfiguration(parent, id),
-    m_isNull(false)
+BuildStepList::BuildStepList(QObject *parent, Core::Id id)
+    : QObject(parent), m_id(id)
 {
-    Q_ASSERT(parent);
-}
-
-BuildStepList::BuildStepList(QObject *parent, BuildStepList *source) :
-    ProjectConfiguration(parent, source),
-    m_isNull(source->m_isNull)
-{
-    setDisplayName(source->displayName());
-    Q_ASSERT(parent);
-    // do not clone the steps here:
-    // The BC is not fully set up yet and thus some of the buildstepfactories
-    // will fail to clone the buildsteps!
-}
-
-BuildStepList::BuildStepList(QObject *parent, const QVariantMap &data) :
-    ProjectConfiguration(parent, Core::Id())
-{
-    Q_ASSERT(parent);
-    m_isNull = !fromMap(data);
+    QTC_ASSERT(parent, return);
+    QTC_ASSERT(parent->parent(), return);
+    m_target = qobject_cast<Target *>(parent->parent());
+    QTC_ASSERT(m_target, return);
 }
 
 BuildStepList::~BuildStepList()
+{
+    clear();
+}
+
+void BuildStepList::clear()
 {
     qDeleteAll(m_steps);
+    m_steps.clear();
 }
 
 QVariantMap BuildStepList::toMap() const
 {
-    QVariantMap map(ProjectConfiguration::toMap());
+    QVariantMap map;
+
+    {
+        // Only written for compatibility reasons within the 4.11 cycle
+        const char CONFIGURATION_ID_KEY[] = "ProjectExplorer.ProjectConfiguration.Id";
+        const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DisplayName";
+        const char DEFAULT_DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DefaultDisplayName";
+        map.insert(QLatin1String(CONFIGURATION_ID_KEY), m_id.toSetting());
+        map.insert(QLatin1String(DISPLAY_NAME_KEY), displayName());
+        map.insert(QLatin1String(DEFAULT_DISPLAY_NAME_KEY), displayName());
+    }
+
     // Save build steps
     map.insert(QString::fromLatin1(STEPS_COUNT_KEY), m_steps.count());
     for (int i = 0; i < m_steps.count(); ++i)
@@ -101,11 +81,6 @@ QVariantMap BuildStepList::toMap() const
     return map;
 }
 
-bool BuildStepList::isNull() const
-{
-    return m_isNull;
-}
-
 int BuildStepList::count() const
 {
     return m_steps.count();
@@ -123,24 +98,29 @@ bool BuildStepList::contains(Core::Id id) const
     });
 }
 
-void BuildStepList::cloneSteps(BuildStepList *source)
+QString BuildStepList::displayName() const
 {
-    Q_ASSERT(source);
-    foreach (BuildStep *originalbs, source->steps()) {
-        IBuildStepFactory *factory(findCloneFactory(this, originalbs));
-        if (!factory)
-            continue;
-        BuildStep *clonebs(factory->clone(this, originalbs));
-        if (clonebs)
-            m_steps.append(clonebs);
+    if (m_id == Constants::BUILDSTEPS_BUILD) {
+        //: Display name of the build build step list. Used as part of the labels in the project window.
+        return tr("Build");
     }
+    if (m_id == Constants::BUILDSTEPS_CLEAN) {
+        //: Display name of the clean build step list. Used as part of the labels in the project window.
+        return tr("Clean");
+    }
+    if (m_id == Constants::BUILDSTEPS_DEPLOY) {
+        //: Display name of the deploy build step list. Used as part of the labels in the project window.
+        return tr("Deploy");
+    }
+    QTC_CHECK(false);
+    return {};
 }
 
 bool BuildStepList::fromMap(const QVariantMap &map)
 {
-    // We need the ID set before trying to restore the steps!
-    if (!ProjectConfiguration::fromMap(map))
-        return false;
+    clear();
+
+    const QList<BuildStepFactory *> factories = BuildStepFactory::allBuildStepFactories();
 
     int maxSteps = map.value(QString::fromLatin1(STEPS_COUNT_KEY), 0).toInt();
     for (int i = 0; i < maxSteps; ++i) {
@@ -149,17 +129,21 @@ bool BuildStepList::fromMap(const QVariantMap &map)
             qWarning() << "No step data found for" << i << "(continuing).";
             continue;
         }
-        IBuildStepFactory *factory = findRestoreFactory(this, bsData);
-        if (!factory) {
-            qWarning() << "No factory for step" << i << "in list" << displayName() << "found (continuing).";
-            continue;
+        bool handled = false;
+        Core::Id stepId = idFromMap(bsData);
+        for (BuildStepFactory *factory : factories) {
+            if (factory->stepId() == stepId) {
+                if (factory->canHandle(this)) {
+                    if (BuildStep *bs = factory->restore(this, bsData)) {
+                        appendStep(bs);
+                        handled = true;
+                    } else {
+                        qWarning() << "Restoration of step" << i << "failed (continuing).";
+                    }
+                }
+            }
         }
-        BuildStep *bs(factory->restore(this, bsData));
-        if (!bs) {
-            qWarning() << "Restoration of step" << i << "failed (continuing).";
-            continue;
-        }
-        insertStep(m_steps.count(), bs);
+        QTC_ASSERT(handled, qDebug() << "No factory for build step" << stepId.toString() << "found.");
     }
     return true;
 }
@@ -169,12 +153,28 @@ QList<BuildStep *> BuildStepList::steps() const
     return m_steps;
 }
 
+BuildStep *BuildStepList::firstStepWithId(Core::Id id) const
+{
+    return Utils::findOrDefault(m_steps, Utils::equal(&BuildStep::id, id));
+}
+
 void BuildStepList::insertStep(int position, BuildStep *step)
 {
     m_steps.insert(position, step);
     emit stepInserted(position);
 }
 
+void BuildStepList::insertStep(int position, Core::Id stepId)
+{
+    for (BuildStepFactory *factory : BuildStepFactory::allBuildStepFactories()) {
+        if (BuildStep *step = factory->create(this, stepId)) {
+            insertStep(position, step);
+            return;
+        }
+    }
+    QTC_ASSERT(false, qDebug() << "No factory for build step" << stepId.toString() << "found.");
+}
+
 bool BuildStepList::removeStep(int position)
 {
     BuildStep *bs = at(position);
@@ -190,7 +190,11 @@ bool BuildStepList::removeStep(int position)
 
 void BuildStepList::moveStepUp(int position)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
+    m_steps.swapItemsAt(position - 1, position);
+#else
     m_steps.swap(position - 1, position);
+#endif
     emit stepMoved(position, position - 1);
 }
 
@@ -199,14 +203,4 @@ BuildStep *BuildStepList::at(int position)
     return m_steps.at(position);
 }
 
-Target *BuildStepList::target() const
-{
-    Q_ASSERT(parent());
-    BuildConfiguration *bc = qobject_cast<BuildConfiguration *>(parent());
-    if (bc)
-        return bc->target();
-    DeployConfiguration *dc = qobject_cast<DeployConfiguration *>(parent());
-    if (dc)
-        return dc->target();
-    return 0;
-}
+} // ProjectExplorer
