@@ -30,11 +30,9 @@
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/documentmanager.h>
-#include <coreplugin/variablechooser.h>
 #include <utils/hostosinfo.h>
 
 #include <QCoreApplication>
-#include <QWidget>
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -43,7 +41,7 @@ namespace Internal {
 
 class ProjectExplorerSettingsWidget : public QWidget
 {
-    Q_OBJECT
+    Q_DECLARE_TR_FUNCTIONS(ProjextExplorer::Internal::ProjectExplorerSettings)
 
 public:
     explicit ProjectExplorerSettingsWidget(QWidget *parent = nullptr);
@@ -57,13 +55,8 @@ public:
     bool useProjectsDirectory();
     void setUseProjectsDirectory(bool v);
 
-    QString buildDirectoryTemplate() const;
-    void setBuildDirectoryTemplate(const QString &bd);
-
 private:
     void slotDirectoryButtonGroupChanged();
-    void resetBuildDirectoryTemplate();
-    void updateBuildDirectoryResetButton();
 
     void setJomVisible(bool);
 
@@ -76,18 +69,24 @@ ProjectExplorerSettingsWidget::ProjectExplorerSettingsWidget(QWidget *parent) :
 {
     m_ui.setupUi(this);
     setJomVisible(Utils::HostOsInfo::isWindowsHost());
+    m_ui.stopBeforeBuildComboBox->addItem(tr("None"), int(StopBeforeBuild::None));
+    m_ui.stopBeforeBuildComboBox->addItem(tr("All"), int(StopBeforeBuild::All));
+    m_ui.stopBeforeBuildComboBox->addItem(tr("Same Project"), int(StopBeforeBuild::SameProject));
+    m_ui.stopBeforeBuildComboBox->addItem(tr("Same Build Directory"),
+                                          int(StopBeforeBuild::SameBuildDir));
+    m_ui.stopBeforeBuildComboBox->addItem(tr("Same Application"),
+                                          int(StopBeforeBuild::SameApp));
+    m_ui.buildBeforeDeployComboBox->addItem(tr("Do Not Build Anything"),
+                                            int(BuildBeforeRunMode::Off));
+    m_ui.buildBeforeDeployComboBox->addItem(tr("Build the Whole Project"),
+                                            int(BuildBeforeRunMode::WholeProject));
+    m_ui.buildBeforeDeployComboBox->addItem(tr("Build Only the Application to Be Run"),
+                                            int(BuildBeforeRunMode::AppOnly));
     m_ui.directoryButtonGroup->setId(m_ui.currentDirectoryRadioButton, UseCurrentDirectory);
     m_ui.directoryButtonGroup->setId(m_ui.directoryRadioButton, UseProjectDirectory);
 
-    connect(m_ui.directoryButtonGroup, static_cast<void (QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked),
+    connect(m_ui.directoryButtonGroup, QOverload<int>::of(&QButtonGroup::buttonClicked),
             this, &ProjectExplorerSettingsWidget::slotDirectoryButtonGroupChanged);
-    connect(m_ui.buildDirectoryResetButton, &QAbstractButton::clicked,
-            this, &ProjectExplorerSettingsWidget::resetBuildDirectoryTemplate);
-    connect(m_ui.buildDirectoryEdit, &QLineEdit::textChanged,
-            this, &ProjectExplorerSettingsWidget::updateBuildDirectoryResetButton);
-
-    auto chooser = new Core::VariableChooser(this);
-    chooser->addSupportedWidget(m_ui.buildDirectoryEdit);
 }
 
 void ProjectExplorerSettingsWidget::setJomVisible(bool v)
@@ -98,44 +97,42 @@ void ProjectExplorerSettingsWidget::setJomVisible(bool v)
 
 ProjectExplorerSettings ProjectExplorerSettingsWidget::settings() const
 {
-    m_settings.buildBeforeDeploy = m_ui.buildProjectBeforeDeployCheckBox->isChecked();
+    m_settings.buildBeforeDeploy = static_cast<BuildBeforeRunMode>(
+                m_ui.buildBeforeDeployComboBox->currentData().toInt());
     m_settings.deployBeforeRun = m_ui.deployProjectBeforeRunCheckBox->isChecked();
     m_settings.saveBeforeBuild = m_ui.saveAllFilesCheckBox->isChecked();
-    m_settings.showCompilerOutput = m_ui.showCompileOutputCheckBox->isChecked();
-    m_settings.showRunOutput = m_ui.showRunOutputCheckBox->isChecked();
-    m_settings.showDebugOutput = m_ui.showDebugOutputCheckBox->isChecked();
-    m_settings.cleanOldAppOutput = m_ui.cleanOldAppOutputCheckBox->isChecked();
-    m_settings.mergeStdErrAndStdOut = m_ui.mergeStdErrAndStdOutCheckBox->isChecked();
-    m_settings.wrapAppOutput = m_ui.wrapAppOutputCheckBox->isChecked();
     m_settings.useJom = m_ui.jomCheckbox->isChecked();
     m_settings.addLibraryPathsToRunEnv = m_ui.addLibraryPathsToRunEnvCheckBox->isChecked();
     m_settings.prompToStopRunControl = m_ui.promptToStopRunControlCheckBox->isChecked();
     m_settings.automaticallyCreateRunConfigurations = m_ui.automaticallyCreateRunConfiguration->isChecked();
-    m_settings.maxAppOutputChars = m_ui.maxAppOutputBox->value();
-    m_settings.maxBuildOutputChars = m_ui.maxBuildOutputBox->value();
-    m_settings.stopBeforeBuild = static_cast<ProjectExplorerSettings::StopBeforeBuild>(m_ui.stopBeforeBuildComboBox->currentIndex());
+    m_settings.stopBeforeBuild = static_cast<StopBeforeBuild>(
+                m_ui.stopBeforeBuildComboBox->currentData().toInt());
+    m_settings.terminalMode = static_cast<TerminalMode>(m_ui.terminalModeComboBox->currentIndex());
+    m_settings.closeSourceFilesWithProject = m_ui.closeSourceFilesCheckBox->isChecked();
+    m_settings.clearIssuesOnRebuild = m_ui.clearIssuesCheckBox->isChecked();
+    m_settings.abortBuildAllOnError = m_ui.abortBuildAllOnErrorCheckBox->isChecked();
+    m_settings.lowBuildPriority = m_ui.lowBuildPriorityCheckBox->isChecked();
     return m_settings;
 }
 
 void ProjectExplorerSettingsWidget::setSettings(const ProjectExplorerSettings  &pes)
 {
     m_settings = pes;
-    m_ui.buildProjectBeforeDeployCheckBox->setChecked(m_settings.buildBeforeDeploy);
+    m_ui.buildBeforeDeployComboBox->setCurrentIndex(
+                m_ui.buildBeforeDeployComboBox->findData(int(m_settings.buildBeforeDeploy)));
     m_ui.deployProjectBeforeRunCheckBox->setChecked(m_settings.deployBeforeRun);
     m_ui.saveAllFilesCheckBox->setChecked(m_settings.saveBeforeBuild);
-    m_ui.showCompileOutputCheckBox->setChecked(m_settings.showCompilerOutput);
-    m_ui.showRunOutputCheckBox->setChecked(m_settings.showRunOutput);
-    m_ui.showDebugOutputCheckBox->setChecked(m_settings.showDebugOutput);
-    m_ui.cleanOldAppOutputCheckBox->setChecked(m_settings.cleanOldAppOutput);
-    m_ui.mergeStdErrAndStdOutCheckBox->setChecked(m_settings.mergeStdErrAndStdOut);
-    m_ui.wrapAppOutputCheckBox->setChecked(m_settings.wrapAppOutput);
     m_ui.jomCheckbox->setChecked(m_settings.useJom);
     m_ui.addLibraryPathsToRunEnvCheckBox->setChecked(m_settings.addLibraryPathsToRunEnv);
     m_ui.promptToStopRunControlCheckBox->setChecked(m_settings.prompToStopRunControl);
     m_ui.automaticallyCreateRunConfiguration->setChecked(m_settings.automaticallyCreateRunConfigurations);
-    m_ui.maxAppOutputBox->setValue(m_settings.maxAppOutputChars);
-    m_ui.maxBuildOutputBox->setValue(m_settings.maxBuildOutputChars);
-    m_ui.stopBeforeBuildComboBox->setCurrentIndex(static_cast<int>(m_settings.stopBeforeBuild));
+    m_ui.stopBeforeBuildComboBox->setCurrentIndex(
+                m_ui.stopBeforeBuildComboBox->findData(int(m_settings.stopBeforeBuild)));
+    m_ui.terminalModeComboBox->setCurrentIndex(static_cast<int>(m_settings.terminalMode));
+    m_ui.closeSourceFilesCheckBox->setChecked(m_settings.closeSourceFilesWithProject);
+    m_ui.clearIssuesCheckBox->setChecked(m_settings.clearIssuesOnRebuild);
+    m_ui.abortBuildAllOnErrorCheckBox->setChecked(m_settings.abortBuildAllOnError);
+    m_ui.lowBuildPriorityCheckBox->setChecked(m_settings.lowBuildPriority);
 }
 
 QString ProjectExplorerSettingsWidget::projectsDirectory() const
@@ -161,41 +158,20 @@ void ProjectExplorerSettingsWidget::setUseProjectsDirectory(bool b)
     }
 }
 
-QString ProjectExplorerSettingsWidget::buildDirectoryTemplate() const
-{
-    return m_ui.buildDirectoryEdit->text();
-}
-
-void ProjectExplorerSettingsWidget::setBuildDirectoryTemplate(const QString &bd)
-{
-    m_ui.buildDirectoryEdit->setText(bd);
-}
-
 void ProjectExplorerSettingsWidget::slotDirectoryButtonGroupChanged()
 {
     bool enable = useProjectsDirectory();
     m_ui.projectsDirectoryPathChooser->setEnabled(enable);
 }
 
-void ProjectExplorerSettingsWidget::resetBuildDirectoryTemplate()
-{
-    setBuildDirectoryTemplate(ProjectExplorerPlugin::defaultBuildDirectoryTemplate());
-}
-
-void ProjectExplorerSettingsWidget::updateBuildDirectoryResetButton()
-{
-    m_ui.buildDirectoryResetButton->setEnabled(buildDirectoryTemplate() != ProjectExplorerPlugin::defaultBuildDirectoryTemplate());
-}
-
 // ------------------ ProjectExplorerSettingsPage
 ProjectExplorerSettingsPage::ProjectExplorerSettingsPage()
 {
     setId(Constants::BUILD_AND_RUN_SETTINGS_PAGE_ID);
-    setDisplayName(tr("General"));
+    setDisplayName(ProjectExplorerSettingsWidget::tr("General"));
     setCategory(Constants::BUILD_AND_RUN_SETTINGS_CATEGORY);
     setDisplayCategory(QCoreApplication::translate("ProjectExplorer", "Build & Run"));
-    setCategoryIcon(Utils::Icon({{":/projectexplorer/images/settingscategory_buildrun.png",
-                    Utils::Theme::PanelTextColorDark}}, Utils::Icon::Tint));
+    setCategoryIconPath(":/projectexplorer/images/settingscategory_buildrun.png");
 }
 
 QWidget *ProjectExplorerSettingsPage::widget()
@@ -205,7 +181,6 @@ QWidget *ProjectExplorerSettingsPage::widget()
         m_widget->setSettings(ProjectExplorerPlugin::projectExplorerSettings());
         m_widget->setProjectsDirectory(Core::DocumentManager::projectsDirectory().toString());
         m_widget->setUseProjectsDirectory(Core::DocumentManager::useProjectsDirectory());
-        m_widget->setBuildDirectoryTemplate(ProjectExplorerPlugin::buildDirectoryTemplate());
     }
     return m_widget;
 }
@@ -215,9 +190,8 @@ void ProjectExplorerSettingsPage::apply()
     if (m_widget) {
         ProjectExplorerPlugin::setProjectExplorerSettings(m_widget->settings());
         Core::DocumentManager::setProjectsDirectory(
-            Utils::FileName::fromString(m_widget->projectsDirectory()));
+            Utils::FilePath::fromString(m_widget->projectsDirectory()));
         Core::DocumentManager::setUseProjectsDirectory(m_widget->useProjectsDirectory());
-        ProjectExplorerPlugin::setBuildDirectoryTemplate(m_widget->buildDirectoryTemplate());
     }
 }
 
@@ -228,5 +202,3 @@ void ProjectExplorerSettingsPage::finish()
 
 } // namespace Internal
 } // namespace ProjectExplorer
-
-#include "projectexplorersettingspage.moc"
