@@ -59,7 +59,7 @@ class RewriterView;
 class NodeInstanceView;
 class TextModifier;
 
-typedef QList<QPair<PropertyName, QVariant> > PropertyListType;
+using PropertyListType = QList<QPair<PropertyName, QVariant> >;
 
 class QMLDESIGNERCORE_EXPORT Model : public QObject
 {
@@ -74,9 +74,9 @@ class QMLDESIGNERCORE_EXPORT Model : public QObject
 public:
     enum ViewNotification { NotifyView, DoNotNotifyView };
 
-    virtual ~Model();
+    ~Model() override;
 
-    static Model *create(TypeName type, int major = 1, int minor = 1, Model *metaInfoPropxyModel = 0);
+    static Model *create(TypeName type, int major = 1, int minor = 1, Model *metaInfoPropxyModel = nullptr);
 
     QUrl fileUrl() const;
     void setFileUrl(const QUrl &url);
@@ -98,8 +98,8 @@ public:
     void changeImports(const QList<Import> &importsToBeAdded, const QList<Import> &importsToBeRemoved);
     void setPossibleImports(const QList<Import> &possibleImports);
     void setUsedImports(const QList<Import> &usedImports);
-    bool hasImport(const Import &import, bool ignoreAlias = true, bool allowHigherVersion = false);
-    bool isImportPossible(const Import &import, bool ignoreAlias = true, bool allowHigherVersion = false);
+    bool hasImport(const Import &import, bool ignoreAlias = true, bool allowHigherVersion = false) const;
+    bool isImportPossible(const Import &import, bool ignoreAlias = true, bool allowHigherVersion = false) const;
     QString pathForImport(const Import &import);
     QStringList importPaths() const;
     Import highestPossibleImport(const QString &importPath);
@@ -116,6 +116,8 @@ public:
     void setTextModifier(TextModifier *textModifier);
     void setDocumentMessages(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings);
 
+    QList<ModelNode> selectedNodes(AbstractView *view) const;
+
 protected:
     Model();
 
