@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,42 +9,38 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "codeassistant.h"
 #include "completionassistprovider.h"
-#include "quickfixassistprovider.h"
 #include "iassistprocessor.h"
-#include "textdocument.h"
 #include "iassistproposal.h"
 #include "iassistproposalmodel.h"
 #include "iassistproposalwidget.h"
 #include "assistinterface.h"
 #include "assistproposalitem.h"
 #include "runner.h"
+#include "textdocumentmanipulator.h"
 
+#include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/completionsettings.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <extensionsystem/pluginmanager.h>
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 #include <QKeyEvent>
@@ -65,9 +61,9 @@ public:
     void configure(TextEditorWidget *editorWidget);
     bool isConfigured() const;
 
-    void invoke(AssistKind kind, IAssistProvider *provider = 0);
+    void invoke(AssistKind kind, IAssistProvider *provider = nullptr);
     void process();
-    void requestProposal(AssistReason reason, AssistKind kind, IAssistProvider *provider = 0);
+    void requestProposal(AssistReason reason, AssistKind kind, IAssistProvider *provider = nullptr);
     void cancelCurrentRequest();
     void invalidateCurrentRequestData();
     void displayProposal(IAssistProposal *newProposal, AssistReason reason);
@@ -78,6 +74,9 @@ public:
     bool hasContext() const;
     void destroyContext();
 
+    QVariant userData() const;
+    void setUserData(const QVariant &data);
+
     CompletionAssistProvider *identifyActivationSequence();
 
     void stopAutomaticProposalTimer();
@@ -86,30 +85,32 @@ public:
     void clearAbortedPosition();
     void updateFromCompletionSettings(const TextEditor::CompletionSettings &settings);
 
-    virtual bool eventFilter(QObject *o, QEvent *e);
+    bool eventFilter(QObject *o, QEvent *e) override;
 
 private:
-    void proposalComputed();
-    void processProposalItem(AssistProposalItem *proposalItem);
+    void processProposalItem(AssistProposalItemInterface *proposalItem);
     void handlePrefixExpansion(const QString &newPrefix);
     void finalizeProposal();
     void explicitlyAborted();
+    bool isDestroyEvent(int key, const QString &keyText);
 
 private:
-    CodeAssistant *q;
-    TextEditorWidget *m_editorWidget;
-    QList<QuickFixAssistProvider *> m_quickFixProviders;
-    Internal::ProcessorRunner *m_requestRunner;
-    IAssistProvider *m_requestProvider;
-    IAssistProcessor *m_asyncProcessor;
-    AssistKind m_assistKind;
-    IAssistProposalWidget *m_proposalWidget;
+    CodeAssistant *q = nullptr;
+    TextEditorWidget *m_editorWidget = nullptr;
+    Internal::ProcessorRunner *m_requestRunner = nullptr;
+    QMetaObject::Connection m_runnerConnection;
+    IAssistProvider *m_requestProvider = nullptr;
+    IAssistProcessor *m_asyncProcessor = nullptr;
+    AssistKind m_assistKind = TextEditor::Completion;
+    IAssistProposalWidget *m_proposalWidget = nullptr;
     QScopedPointer<IAssistProposal> m_proposal;
-    bool m_receivedContentWhileWaiting;
+    bool m_receivedContentWhileWaiting = false;
+    bool m_proposalItemProcessed = false;
     QTimer m_automaticProposalTimer;
     CompletionSettings m_settings;
-    int m_abortedBasePosition;
+    int m_abortedBasePosition = -1;
     static const QChar m_null;
+    QVariant m_userData;
 };
 
 // --------------------
@@ -119,19 +120,12 @@ const QChar CodeAssistantPrivate::m_null;
 
 CodeAssistantPrivate::CodeAssistantPrivate(CodeAssistant *assistant)
     : q(assistant)
-    , m_editorWidget(0)
-    , m_requestRunner(0)
-    , m_requestProvider(0)
-    , m_asyncProcessor(0)
-    , m_assistKind(TextEditor::Completion)
-    , m_proposalWidget(0)
-    , m_receivedContentWhileWaiting(false)
-    , m_abortedBasePosition(-1)
 {
     m_automaticProposalTimer.setSingleShot(true);
     connect(&m_automaticProposalTimer, &QTimer::timeout,
             this, &CodeAssistantPrivate::automaticProposalTimeout);
 
+    m_settings = TextEditorSettings::completionSettings();
     connect(TextEditorSettings::instance(), &TextEditorSettings::completionSettingsChanged,
             this, &CodeAssistantPrivate::updateFromCompletionSettings);
 
@@ -141,29 +135,13 @@ CodeAssistantPrivate::CodeAssistantPrivate(CodeAssistant *assistant)
 
 void CodeAssistantPrivate::configure(TextEditorWidget *editorWidget)
 {
-    // @TODO: There's a list of providers but currently only the first one is used. Perhaps we
-    // should implement a truly mechanism to support multiple providers for an editor (either
-    // merging or not proposals) or just leave it as not extensible and store directly the one
-    // completion and quick-fix provider (getting rid of the list).
-
     m_editorWidget = editorWidget;
-    m_quickFixProviders = ExtensionSystem::PluginManager::getObjects<QuickFixAssistProvider>();
-
-    Core::Id editorId = m_editorWidget->textDocument()->id();
-    auto it = m_quickFixProviders.begin();
-    while (it != m_quickFixProviders.end()) {
-        if ((*it)->supportsEditor(editorId))
-            ++it;
-        else
-            it = m_quickFixProviders.erase(it);
-    }
-
     m_editorWidget->installEventFilter(this);
 }
 
 bool CodeAssistantPrivate::isConfigured() const
 {
-    return m_editorWidget != 0;
+    return m_editorWidget != nullptr;
 }
 
 void CodeAssistantPrivate::invoke(AssistKind kind, IAssistProvider *provider)
@@ -216,23 +194,30 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
     if (m_editorWidget->hasBlockSelection())
         return; // TODO
 
+    if (m_proposalItemProcessed
+            && reason == IdleEditor
+            && m_assistKind == TextEditor::Completion
+            && !identifyActivationSequence()) {
+        return;
+    }
+
     if (!provider) {
         if (kind == Completion)
             provider = m_editorWidget->textDocument()->completionAssistProvider();
-        else if (!m_quickFixProviders.isEmpty())
-            provider = m_quickFixProviders.at(0);
+        else
+            provider = m_editorWidget->textDocument()->quickFixAssistProvider();
 
         if (!provider)
             return;
     }
 
-    m_assistKind = kind;
-    IAssistProcessor *processor = provider->createProcessor();
     AssistInterface *assistInterface = m_editorWidget->createAssistInterface(kind, reason);
-    if (!assistInterface) {
-        delete processor;
+    if (!assistInterface)
         return;
-    }
+
+    m_assistKind = kind;
+    m_requestProvider = provider;
+    IAssistProcessor *processor = provider->createProcessor();
 
     switch (provider->runType()) {
     case IAssistProvider::Synchronous: {
@@ -245,16 +230,22 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
         if (IAssistProposal *newProposal = processor->immediateProposal(assistInterface))
             displayProposal(newProposal, reason);
 
-        m_requestProvider = provider;
         m_requestRunner = new ProcessorRunner;
+        m_runnerConnection = connect(m_requestRunner, &ProcessorRunner::finished,
+                                     this, [this, reason](){
+            // Since the request runner is a different thread, there's still a gap in which the
+            // queued signal could be processed after an invalidation of the current request.
+            if (!m_requestRunner || m_requestRunner != sender())
+                return;
+
+            IAssistProposal *proposal = m_requestRunner->proposal();
+            invalidateCurrentRequestData();
+            displayProposal(proposal, reason);
+            emit q->finished();
+        });
         connect(m_requestRunner, &ProcessorRunner::finished,
-                this, &CodeAssistantPrivate::proposalComputed);
-        connect(m_requestRunner, &ProcessorRunner::finished,
-                m_requestRunner, &QObject::deleteLater);
-        connect(m_requestRunner, &ProcessorRunner::finished,
-                q, &CodeAssistant::finished);
+                m_requestRunner, &ProcessorRunner::deleteLater);
         assistInterface->prepareForAsyncUse();
-        m_requestRunner->setReason(reason);
         m_requestRunner->setProcessor(processor);
         m_requestRunner->setAssistInterface(assistInterface);
         m_requestRunner->start();
@@ -262,30 +253,30 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
     }
     case IAssistProvider::Asynchronous: {
         processor->setAsyncCompletionAvailableHandler(
-            [this, processor, reason](IAssistProposal *newProposal){
-                if (m_asyncProcessor != processor) {
-                    delete newProposal->model();
+            [this, reason](IAssistProposal *newProposal){
+                if (m_asyncProcessor && m_asyncProcessor->needsRestart() && m_receivedContentWhileWaiting) {
                     delete newProposal;
-                    return;
+                    m_receivedContentWhileWaiting = false;
+                    invalidateCurrentRequestData();
+                    requestProposal(reason, m_assistKind, m_requestProvider);
+                } else {
+                    invalidateCurrentRequestData();
+                    displayProposal(newProposal, reason);
+
+                    emit q->finished();
                 }
-
-                invalidateCurrentRequestData();
-                QTC_CHECK(newProposal);
-                displayProposal(newProposal, reason);
-
-                emit q->finished();
         });
 
         // If there is a proposal, nothing asynchronous happened...
         if (IAssistProposal *newProposal = processor->perform(assistInterface)) {
             displayProposal(newProposal, reason);
             delete processor;
-        } else if (!processor->performWasApplicable()) {
+        } else if (!processor->running()) {
             delete processor;
+        } else { // ...async request was triggered
+            m_asyncProcessor = processor;
         }
 
-        // ...otherwise the async request was triggered
-        m_asyncProcessor = processor;
         break;
     }
     } // switch
@@ -295,51 +286,59 @@ void CodeAssistantPrivate::cancelCurrentRequest()
 {
     if (m_requestRunner) {
         m_requestRunner->setDiscardProposal(true);
-        disconnect(m_requestRunner, &ProcessorRunner::finished,
-                   this, &CodeAssistantPrivate::proposalComputed);
+        disconnect(m_runnerConnection);
     }
+    if (m_asyncProcessor)
+        m_asyncProcessor->cancel();
     invalidateCurrentRequestData();
 }
 
-void CodeAssistantPrivate::proposalComputed()
-{
-    // Since the request runner is a different thread, there's still a gap in which the queued
-    // signal could be processed after an invalidation of the current request.
-    if (!m_requestRunner || m_requestRunner != sender())
-        return;
-
-    IAssistProposal *newProposal = m_requestRunner->proposal();
-    AssistReason reason = m_requestRunner->reason();
-    invalidateCurrentRequestData();
-    displayProposal(newProposal, reason);
-}
-
 void CodeAssistantPrivate::displayProposal(IAssistProposal *newProposal, AssistReason reason)
 {
     if (!newProposal)
         return;
 
+    // TODO: The proposal should own the model until someone takes it explicitly away.
     QScopedPointer<IAssistProposal> proposalCandidate(newProposal);
 
+    bool destroyCurrentContext = false;
     if (isDisplayingProposal()) {
         if (!m_proposal->isFragile())
             return;
-        destroyContext();
+        destroyCurrentContext = true;
     }
 
     int basePosition = proposalCandidate->basePosition();
-    if (m_editorWidget->position() < basePosition)
+    if (m_editorWidget->position() < basePosition) {
+        if (destroyCurrentContext)
+            destroyContext();
         return;
+    }
 
-    if (m_abortedBasePosition == basePosition && reason != ExplicitlyInvoked)
+    if (m_abortedBasePosition == basePosition && reason != ExplicitlyInvoked) {
+        if (destroyCurrentContext)
+            destroyContext();
         return;
+    }
+
+    const QString prefix = m_editorWidget->textAt(basePosition,
+                                                  m_editorWidget->position() - basePosition);
+    if (!newProposal->hasItemsToPropose(prefix, reason)) {
+        if (newProposal->isCorrective(m_editorWidget))
+            newProposal->makeCorrection(m_editorWidget);
+        return;
+    }
+
+    if (destroyCurrentContext)
+        destroyContext();
 
     clearAbortedPosition();
     m_proposal.reset(proposalCandidate.take());
 
-    if (m_proposal->isCorrective())
+    if (m_proposal->isCorrective(m_editorWidget))
         m_proposal->makeCorrection(m_editorWidget);
 
+    m_editorWidget->keepAutoCompletionHighlight(true);
     basePosition = m_proposal->basePosition();
     m_proposalWidget = m_proposal->createWidget();
     connect(m_proposalWidget, &QObject::destroyed,
@@ -353,30 +352,55 @@ void CodeAssistantPrivate::displayProposal(IAssistProposal *newProposal, AssistR
     m_proposalWidget->setAssistant(q);
     m_proposalWidget->setReason(reason);
     m_proposalWidget->setKind(m_assistKind);
+    m_proposalWidget->setBasePosition(basePosition);
     m_proposalWidget->setUnderlyingWidget(m_editorWidget);
     m_proposalWidget->setModel(m_proposal->model());
     m_proposalWidget->setDisplayRect(m_editorWidget->cursorRect(basePosition));
-    if (m_receivedContentWhileWaiting)
-        m_proposalWidget->setIsSynchronized(false);
-    else
-        m_proposalWidget->setIsSynchronized(true);
-    m_proposalWidget->showProposal(m_editorWidget->textAt(
-                                       basePosition,
-                                       m_editorWidget->position() - basePosition));
+    m_proposalWidget->setIsSynchronized(!m_receivedContentWhileWaiting);
+    m_proposalWidget->showProposal(prefix);
 }
 
-void CodeAssistantPrivate::processProposalItem(AssistProposalItem *proposalItem)
+void CodeAssistantPrivate::processProposalItem(AssistProposalItemInterface *proposalItem)
 {
     QTC_ASSERT(m_proposal, return);
-    proposalItem->apply(m_editorWidget, m_proposal->basePosition());
+    TextDocumentManipulator manipulator(m_editorWidget);
+    proposalItem->apply(manipulator, m_proposal->basePosition());
     destroyContext();
-    process();
+    if (!proposalItem->isSnippet())
+        process();
+    m_editorWidget->encourageApply();
+    m_proposalItemProcessed = true;
+
+    auto connection = std::make_shared<QMetaObject::Connection>();
+    *connection = connect(m_editorWidget->textDocument(),
+                          &Core::IDocument::contentsChanged,
+                          this, [this, connection] {
+        m_proposalItemProcessed = false;
+        disconnect(*connection);
+    });
 }
 
 void CodeAssistantPrivate::handlePrefixExpansion(const QString &newPrefix)
 {
     QTC_ASSERT(m_proposal, return);
-    const int currentPosition = m_editorWidget->position();
+
+    QTextCursor cursor(m_editorWidget->document());
+    cursor.setPosition(m_proposal->basePosition());
+    cursor.movePosition(QTextCursor::EndOfWord);
+
+    int currentPosition = m_editorWidget->position();
+    const QString textAfterCursor = m_editorWidget->textAt(currentPosition,
+                                                       cursor.position() - currentPosition);
+    if (!textAfterCursor.startsWith(newPrefix)) {
+        if (newPrefix.indexOf(textAfterCursor, currentPosition - m_proposal->basePosition()) >= 0)
+            currentPosition = cursor.position();
+        const QStringRef prefixAddition =
+                newPrefix.midRef(currentPosition - m_proposal->basePosition());
+        // If remaining string starts with the prefix addition
+        if (textAfterCursor.startsWith(prefixAddition))
+            currentPosition += prefixAddition.size();
+    }
+
     m_editorWidget->setCursorPosition(m_proposal->basePosition());
     m_editorWidget->replace(currentPosition - m_proposal->basePosition(), newPrefix);
     notifyChange();
@@ -386,46 +410,52 @@ void CodeAssistantPrivate::finalizeProposal()
 {
     stopAutomaticProposalTimer();
     m_proposal.reset();
-    m_proposalWidget = 0;
+    m_proposalWidget = nullptr;
     if (m_receivedContentWhileWaiting)
         m_receivedContentWhileWaiting = false;
 }
 
 bool CodeAssistantPrivate::isDisplayingProposal() const
 {
-    return m_proposalWidget != 0;
+    return m_proposalWidget != nullptr;
 }
 
 bool CodeAssistantPrivate::isWaitingForProposal() const
 {
-    return m_requestRunner != 0 || m_asyncProcessor != 0;
+    return m_requestRunner != nullptr || m_asyncProcessor != nullptr;
 }
 
 void CodeAssistantPrivate::invalidateCurrentRequestData()
 {
-    m_asyncProcessor = 0;
-    m_requestRunner = 0;
-    m_requestProvider = 0;
+    m_asyncProcessor = nullptr;
+    m_requestRunner = nullptr;
+    m_requestProvider = nullptr;
 }
 
 CompletionAssistProvider *CodeAssistantPrivate::identifyActivationSequence()
 {
-    CompletionAssistProvider *completionProvider = m_editorWidget->textDocument()->completionAssistProvider();
-    if (!completionProvider)
-        return 0;
+    auto checkActivationSequence = [this](CompletionAssistProvider *provider) {
+        if (!provider)
+            return false;
+        const int length = provider->activationCharSequenceLength();
+        if (!length)
+            return false;
+        QString sequence = m_editorWidget->textAt(m_editorWidget->position() - length, length);
+        // In pretty much all cases the sequence will have the appropriate length. Only in the
+        // case of typing the very first characters in the document for providers that request a
+        // length greater than 1 (currently only C++, which specifies 3), the sequence needs to
+        // be prepended so it has the expected length.
+        const int lengthDiff = length - sequence.length();
+        for (int j = 0; j < lengthDiff; ++j)
+            sequence.prepend(m_null);
+        return provider->isActivationCharSequence(sequence);
+    };
 
-    const int length = completionProvider->activationCharSequenceLength();
-    if (length == 0)
-        return 0;
-    QString sequence = m_editorWidget->textAt(m_editorWidget->position() - length, length);
-    // In pretty much all cases the sequence will have the appropriate length. Only in the
-    // case of typing the very first characters in the document for providers that request a
-    // length greater than 1 (currently only C++, which specifies 3), the sequence needs to
-    // be prepended so it has the expected length.
-    const int lengthDiff = length - sequence.length();
-    for (int j = 0; j < lengthDiff; ++j)
-        sequence.prepend(m_null);
-    return completionProvider->isActivationCharSequence(sequence) ? completionProvider : 0;
+    auto provider = {
+        m_editorWidget->textDocument()->completionAssistProvider(),
+        m_editorWidget->textDocument()->functionHintAssistProvider()
+    };
+    return Utils::findOrDefault(provider, checkActivationSequence);
 }
 
 void CodeAssistantPrivate::notifyChange()
@@ -436,12 +466,13 @@ void CodeAssistantPrivate::notifyChange()
         QTC_ASSERT(m_proposal, return);
         if (m_editorWidget->position() < m_proposal->basePosition()) {
             destroyContext();
-        } else {
+        } else if (m_proposal->supportsPrefix()) {
             m_proposalWidget->updateProposal(
                 m_editorWidget->textAt(m_proposal->basePosition(),
                                      m_editorWidget->position() - m_proposal->basePosition()));
-            if (m_proposal->isFragile())
-                startAutomaticProposalTimer();
+        } else {
+            destroyContext();
+            requestProposal(ExplicitlyInvoked, m_assistKind, m_requestProvider);
         }
     }
 }
@@ -458,6 +489,7 @@ void CodeAssistantPrivate::destroyContext()
     if (isWaitingForProposal()) {
         cancelCurrentRequest();
     } else if (isDisplayingProposal()) {
+        m_editorWidget->keepAutoCompletionHighlight(false);
         m_proposalWidget->closeProposal();
         disconnect(m_proposalWidget, &QObject::destroyed,
                    this, &CodeAssistantPrivate::finalizeProposal);
@@ -465,6 +497,16 @@ void CodeAssistantPrivate::destroyContext()
     }
 }
 
+QVariant CodeAssistantPrivate::userData() const
+{
+    return m_userData;
+}
+
+void CodeAssistantPrivate::setUserData(const QVariant &data)
+{
+    m_userData = data;
+}
+
 void CodeAssistantPrivate::startAutomaticProposalTimer()
 {
     if (m_settings.m_completionTrigger == AutomaticCompletion)
@@ -503,31 +545,31 @@ void CodeAssistantPrivate::clearAbortedPosition()
     m_abortedBasePosition = -1;
 }
 
+bool CodeAssistantPrivate::isDestroyEvent(int key, const QString &keyText)
+{
+    if (keyText.isEmpty())
+        return key != Qt::LeftArrow && key != Qt::RightArrow && key != Qt::Key_Shift;
+    if (auto provider = qobject_cast<CompletionAssistProvider *>(m_requestProvider))
+        return !provider->isContinuationChar(keyText.at(0));
+    return false;
+}
+
 bool CodeAssistantPrivate::eventFilter(QObject *o, QEvent *e)
 {
-    Q_UNUSED(o);
+    Q_UNUSED(o)
 
     if (isWaitingForProposal()) {
         QEvent::Type type = e->type();
         if (type == QEvent::FocusOut) {
             destroyContext();
         } else if (type == QEvent::KeyPress) {
-            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(e);
+            auto keyEvent = static_cast<QKeyEvent *>(e);
             const QString &keyText = keyEvent->text();
 
-            CompletionAssistProvider *completionProvider = 0;
-            if ((keyText.isEmpty()
-                 && keyEvent->key() != Qt::LeftArrow
-                 && keyEvent->key() != Qt::RightArrow
-                 && keyEvent->key() != Qt::Key_Shift)
-                || (!keyText.isEmpty()
-                    && (((completionProvider = dynamic_cast<CompletionAssistProvider *>(m_requestProvider))
-                            ? !completionProvider->isContinuationChar(keyText.at(0))
-                            : false)))) {
+            if (isDestroyEvent(keyEvent->key(), keyText))
                 destroyContext();
-            } else if (!keyText.isEmpty() && !m_receivedContentWhileWaiting) {
+            else if (!keyText.isEmpty() && !m_receivedContentWhileWaiting)
                 m_receivedContentWhileWaiting = true;
-            }
         }
     }
 
@@ -552,11 +594,6 @@ void CodeAssistant::configure(TextEditorWidget *editorWidget)
     d->configure(editorWidget);
 }
 
-void CodeAssistant::updateFromCompletionSettings(const CompletionSettings &settings)
-{
-    d->updateFromCompletionSettings(settings);
-}
-
 void CodeAssistant::process()
 {
     d->process();
@@ -577,6 +614,16 @@ void CodeAssistant::destroyContext()
     d->destroyContext();
 }
 
+QVariant CodeAssistant::userData() const
+{
+    return d->userData();
+}
+
+void CodeAssistant::setUserData(const QVariant &data)
+{
+    d->setUserData(data);
+}
+
 void CodeAssistant::invoke(AssistKind kind, IAssistProvider *provider)
 {
     d->invoke(kind, provider);
