@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef CALLGRINDNAMEDELEGATE_H
-#define CALLGRINDNAMEDELEGATE_H
+#pragma once
 
 #include <QStyledItemDelegate>
 
@@ -34,13 +33,11 @@ namespace Internal {
 class NameDelegate : public QStyledItemDelegate
 {
 public:
-    explicit NameDelegate(QObject *parent = 0);
+    explicit NameDelegate(QObject *parent = nullptr);
 
     void paint(QPainter *painter, const QStyleOptionViewItem &option,
-               const QModelIndex &index) const;
+               const QModelIndex &index) const override;
 };
 
 } // namespace Internal
 } // namespace Valgrind
-
-#endif // CALLGRINDNAMEDELEGATE_H
