@@ -52,12 +52,10 @@ public:
                       const QList<XmlProtocol::Error> &errors);
     static void maybeShow(MemcheckErrorView *view);
 
-private slots:
-    void validate();
-
 private:
-    void accept();
-    void reject();
+    void validate();
+    void accept() override;
+    void reject() override;
 
     MemcheckErrorView *m_view;
     ValgrindBaseSettings *m_settings;
